# -*- coding: utf-8 -*-
"""
bots-grammar: xml
xml-schema: UBL-Invoice-2.1.xsd
Author: Ludovic Watteaux
"""
from bots.botsconfig import ID, MIN, MAX, LEVEL, QUERIES

from .UBL21records import (version, ext, cbc, cac, ccts, qdt, udt, xsi, invoice, recorddefs)
from .UBL21sequences import sequence


xmlns = invoice
ubltype = xmlns.split(':')[-1].replace('-2}', '')


syntax = {
    'merged': False,
    'indented': True,
    'xmlns': xmlns,
    'ext': ext,
    'cbc': cbc,
    'cac': cac,
    'ccts': ccts,
    'qdt': qdt,
    'udt': udt,
    'xsi': xsi,
    'ubltype': ubltype,
    'schema_version': version,
    'schema_location': xmlns.strip('{}')+' http://docs.oasis-open.org/ubl/os-UBL-2.1/xsd/maindoc/UBL-Invoice-2.1.xsd',
    'namespace_prefixes': [
        ('', xmlns.strip('{}')),  # xmlns
        ('ext', ext.strip('{}')),
        ('cbc', cbc.strip('{}')),
        ('cac', cac.strip('{}')),
        ('ccts', ccts.strip('{}')),
        ('xsi', xsi.strip('{}')),
    ],
}


structure = [
{ID: xmlns+'Invoice', MIN: 1, MAX: 1,
    QUERIES: {
        'reference': (
            {'BOTSID': xmlns+'Invoice'},
            {'BOTSID': cbc+'ID', 'BOTSCONTENT': None},
        ),
        'doctype': (
            {'BOTSID': xmlns+'Invoice'},
            {'BOTSID': cbc+'InvoiceTypeCode', 'BOTSCONTENT': None},
        ),
    },
    LEVEL: [
    {ID: ext+'UBLExtensions', MIN: 0, MAX: 1, LEVEL: [
        {ID: ext+'UBLExtension', MIN: 1, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: ext+'ExtensionAgencyID', MIN: 0, MAX: 1},
            {ID: ext+'ExtensionAgencyName', MIN: 0, MAX: 1},
            {ID: ext+'ExtensionVersionID', MIN: 0, MAX: 1},
            {ID: ext+'ExtensionAgencyURI', MIN: 0, MAX: 1},
            {ID: ext+'ExtensionURI', MIN: 0, MAX: 1},
            {ID: ext+'ExtensionReasonCode', MIN: 0, MAX: 1},
            {ID: ext+'ExtensionReason', MIN: 0, MAX: 1},
            {ID: ext+'ExtensionContent', MIN: 1, MAX: 1, LEVEL: [
                {ID: xmlns+'CategoryCode', MIN: 0, MAX: 1},
            ]},
        ]},
    ]},
    {ID: cbc+'UBLVersionID', MIN: 0, MAX: 1},
    {ID: cbc+'CustomizationID', MIN: 0, MAX: 1},
    {ID: cbc+'ProfileID', MIN: 0, MAX: 1},
    {ID: cbc+'ProfileExecutionID', MIN: 0, MAX: 1},
    {ID: cbc+'ID', MIN: 1, MAX: 1},
    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
    {ID: cbc+'UUID', MIN: 0, MAX: 1},
    {ID: cbc+'IssueDate', MIN: 1, MAX: 1},
    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
    {ID: cbc+'DueDate', MIN: 0, MAX: 1},
    {ID: cbc+'InvoiceTypeCode', MIN: 0, MAX: 1},
    {ID: cbc+'Note', MIN: 0, MAX: 99999},
    {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
    {ID: cbc+'DocumentCurrencyCode', MIN: 0, MAX: 1},
    {ID: cbc+'TaxCurrencyCode', MIN: 0, MAX: 1},
    {ID: cbc+'PricingCurrencyCode', MIN: 0, MAX: 1},
    {ID: cbc+'PaymentCurrencyCode', MIN: 0, MAX: 1},
    {ID: cbc+'PaymentAlternativeCurrencyCode', MIN: 0, MAX: 1},
    {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
    {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
    {ID: cbc+'LineCountNumeric', MIN: 0, MAX: 1},
    {ID: cbc+'BuyerReference', MIN: 0, MAX: 1},
    {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'StartDate', MIN: 0, MAX: 1},
        {ID: cbc+'StartTime', MIN: 0, MAX: 1},
        {ID: cbc+'EndDate', MIN: 0, MAX: 1},
        {ID: cbc+'EndTime', MIN: 0, MAX: 1},
        {ID: cbc+'DurationMeasure', MIN: 0, MAX: 1},
        {ID: cbc+'DescriptionCode', MIN: 0, MAX: 99999},
        {ID: cbc+'Description', MIN: 0, MAX: 99999},
    ]},
    {ID: cac+'OrderReference', MIN: 0, MAX: 1, LEVEL: [
        {ID: cbc+'ID', MIN: 1, MAX: 1},
        {ID: cbc+'SalesOrderID', MIN: 0, MAX: 1},
        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'UUID', MIN: 0, MAX: 1},
        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
        {ID: cbc+'CustomerReference', MIN: 0, MAX: 1},
        {ID: cbc+'OrderTypeCode', MIN: 0, MAX: 1},
        {ID: cac+'DocumentReference', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                            ]},
                                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                ]},
                                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                            ]},
                                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                ]},
                                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                                ]},
                                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                    ]},
                                                ]},
                                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                    ]},
                                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                    ]},
                                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                            ]},
                                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                ]},
                                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                                ]},
                                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                    ]},
                                                ]},
                                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                    ]},
                                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                    ]},
                                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                                ]},
                                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                    ]},
                                                ]},
                                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                    ]},
                                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                    ]},
                                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                    ]},
                                                ]},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                    ]},
                                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                                    ]},
                                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                        ]},
                                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                                ]},
                                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                        ]},
                                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                                    ]},
                                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                            ]},
                                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                ]},
                                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                            ]},
                                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                ]},
                                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                            ]},
                                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                ]},
                                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                            ]},
                                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                ]},
                                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                            ]},
                                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                ]},
                                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                            ]},
                                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                                ]},
                                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'BillingReference', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cac+'InvoiceDocumentReference', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'SelfBilledInvoiceDocumentReference', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'CreditNoteDocumentReference', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'SelfBilledCreditNoteDocumentReference', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'DebitNoteDocumentReference', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'ReminderDocumentReference', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'AdditionalDocumentReference', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                                            {ID: cbc+'District', MIN: 0, MAX: 1},
                                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                        ]},
                                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                                {ID: cac+'Party', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            ]},
                                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                            {ID: cac+'Party', MIN: 1, MAX: 1},
                                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'BillingReferenceLine', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'Amount', MIN: 0, MAX: 1},
            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxCategory', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                            {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                            {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                    {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                    {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                        {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                        {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                        {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                        {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                        {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                        {ID: cac+'PayerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Content', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                        {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancingParty', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                    ]},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'DespatchDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 1, MAX: 1},
        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'UUID', MIN: 0, MAX: 1},
        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'URI', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
            ]},
        ]},
        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'ReceiptDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 1, MAX: 1},
        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'UUID', MIN: 0, MAX: 1},
        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'URI', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
            ]},
        ]},
        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'StatementDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 1, MAX: 1},
        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'UUID', MIN: 0, MAX: 1},
        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'URI', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
            ]},
        ]},
        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'OriginatorDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 1, MAX: 1},
        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'UUID', MIN: 0, MAX: 1},
        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'URI', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
            ]},
        ]},
        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 1, MAX: 1},
        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'UUID', MIN: 0, MAX: 1},
        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'URI', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
            ]},
        ]},
        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'AdditionalDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 1, MAX: 1},
        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'UUID', MIN: 0, MAX: 1},
        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'URI', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
            ]},
        ]},
        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                                        {ID: cbc+'District', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                    ]},
                                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                            {ID: cac+'Party', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        ]},
                                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                        {ID: cac+'Party', MIN: 1, MAX: 1},
                                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'ProjectReference', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 1, MAX: 1},
        {ID: cbc+'UUID', MIN: 0, MAX: 1},
        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
        {ID: cac+'WorkPhaseReference', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'WorkPhaseCode', MIN: 0, MAX: 1},
            {ID: cbc+'WorkPhase', MIN: 0, MAX: 99999},
            {ID: cbc+'ProgressPercent', MIN: 0, MAX: 1},
            {ID: cbc+'StartDate', MIN: 0, MAX: 1},
            {ID: cbc+'EndDate', MIN: 0, MAX: 1},
            {ID: cac+'WorkOrderDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'Signature', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 1, MAX: 1},
        {ID: cbc+'Note', MIN: 0, MAX: 99999},
        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
        {ID: cbc+'CanonicalizationMethod', MIN: 0, MAX: 1},
        {ID: cbc+'SignatureMethod', MIN: 0, MAX: 1},
        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'DigitalSignatureAttachment', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'URI', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
            ]},
        ]},
        {ID: cac+'OriginalDocumentReference', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'AccountingSupplierParty', MIN: 1, MAX: 1, LEVEL: [
        {ID: cbc+'CustomerAssignedAccountID', MIN: 0, MAX: 1},
        {ID: cbc+'AdditionalAccountID', MIN: 0, MAX: 99999},
        {ID: cbc+'DataSendingCapability', MIN: 0, MAX: 1},
        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'DespatchContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'AccountingContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
    ]},
    {ID: cac+'AccountingCustomerParty', MIN: 1, MAX: 1, LEVEL: [
        {ID: cbc+'CustomerAssignedAccountID', MIN: 0, MAX: 1},
        {ID: cbc+'SupplierAssignedAccountID', MIN: 0, MAX: 1},
        {ID: cbc+'AdditionalAccountID', MIN: 0, MAX: 99999},
        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'DeliveryContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'AccountingContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'BuyerContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
    ]},
    {ID: cac+'PayeeParty', MIN: 0, MAX: 1, LEVEL: [
        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
        ]},
        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'Name', MIN: 1, MAX: 1},
        ]},
        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
        ]},
        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
            {ID: cbc+'Floor', MIN: 0, MAX: 1},
            {ID: cbc+'Room', MIN: 0, MAX: 1},
            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
            {ID: cbc+'Department', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
            {ID: cbc+'CityName', MIN: 0, MAX: 1},
            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
            {ID: cbc+'Region', MIN: 0, MAX: 1},
            {ID: cbc+'District', MIN: 0, MAX: 1},
            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Line', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
        ]},
        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
            {ID: cbc+'Title', MIN: 0, MAX: 1},
            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
            {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
        ]},
    ]},
    {ID: cac+'BuyerCustomerParty', MIN: 0, MAX: 1, LEVEL: [
        {ID: cbc+'CustomerAssignedAccountID', MIN: 0, MAX: 1},
        {ID: cbc+'SupplierAssignedAccountID', MIN: 0, MAX: 1},
        {ID: cbc+'AdditionalAccountID', MIN: 0, MAX: 99999},
        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'DeliveryContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'AccountingContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'BuyerContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
    ]},
    {ID: cac+'SellerSupplierParty', MIN: 0, MAX: 1, LEVEL: [
        {ID: cbc+'CustomerAssignedAccountID', MIN: 0, MAX: 1},
        {ID: cbc+'AdditionalAccountID', MIN: 0, MAX: 99999},
        {ID: cbc+'DataSendingCapability', MIN: 0, MAX: 1},
        {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'DespatchContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'AccountingContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
    ]},
    {ID: cac+'TaxRepresentativeParty', MIN: 0, MAX: 1, LEVEL: [
        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
        ]},
        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'Name', MIN: 1, MAX: 1},
        ]},
        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
        ]},
        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
            {ID: cbc+'Floor', MIN: 0, MAX: 1},
            {ID: cbc+'Room', MIN: 0, MAX: 1},
            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
            {ID: cbc+'Department', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
            {ID: cbc+'CityName', MIN: 0, MAX: 1},
            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
            {ID: cbc+'Region', MIN: 0, MAX: 1},
            {ID: cbc+'District', MIN: 0, MAX: 1},
            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Line', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
        ]},
        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                {ID: cac+'Party', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
            {ID: cbc+'Title', MIN: 0, MAX: 1},
            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                                    {ID: cbc+'District', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                                ]},
                                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                ]},
                                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                        {ID: cac+'Party', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    ]},
                                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                    {ID: cac+'Party', MIN: 1, MAX: 1},
                                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
            {ID: cac+'Party', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'SellerContact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cac+'NotaryParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'AgentParty', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
        ]},
    ]},
    {ID: cac+'Delivery', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 0, MAX: 1},
        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
        {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
        {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
        {ID: cbc+'ActualDeliveryDate', MIN: 0, MAX: 1},
        {ID: cbc+'ActualDeliveryTime', MIN: 0, MAX: 1},
        {ID: cbc+'LatestDeliveryDate', MIN: 0, MAX: 1},
        {ID: cbc+'LatestDeliveryTime', MIN: 0, MAX: 1},
        {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
        {ID: cbc+'TrackingID', MIN: 0, MAX: 1},
        {ID: cac+'DeliveryAddress', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
            {ID: cbc+'Floor', MIN: 0, MAX: 1},
            {ID: cbc+'Room', MIN: 0, MAX: 1},
            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
            {ID: cbc+'Department', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
            {ID: cbc+'CityName', MIN: 0, MAX: 1},
            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
            {ID: cbc+'Region', MIN: 0, MAX: 1},
            {ID: cbc+'District', MIN: 0, MAX: 1},
            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Line', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
        ]},
        {ID: cac+'AlternativeDeliveryLocation', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
        ]},
        {ID: cac+'RequestedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'PromisedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'EstimatedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'CarrierParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'DeliveryParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'NotifyParty', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'Despatch', MIN: 0, MAX: 1, LEVEL: sequence(cac+'Despatch')},
        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
            {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
            {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
            {ID: cbc+'Amount', MIN: 0, MAX: 1},
            {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                    {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                    {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                        {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                        {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                        {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                        {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                        {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                        {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Content', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                        {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                        {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'MinimumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'BatchQuantity', MIN: 1, MAX: 1},
            {ID: cbc+'ConsumerUnitQuantity', MIN: 0, MAX: 1},
            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
        ]},
        {ID: cac+'MaximumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DeliveryUnitType')},
        {ID: cac+'Shipment', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
            {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
            {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
            {ID: cbc+'Information', MIN: 0, MAX: 99999},
            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
            {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
            {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
            {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
            {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
            {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
            {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
            {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
            {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
            {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
            {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
            {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
            {ID: cac+'Consignment', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CarrierAssignedID', MIN: 0, MAX: 1},
                {ID: cbc+'ConsigneeAssignedID', MIN: 0, MAX: 1},
                {ID: cbc+'ConsignorAssignedID', MIN: 0, MAX: 1},
                {ID: cbc+'FreightForwarderAssignedID', MIN: 0, MAX: 1},
                {ID: cbc+'BrokerAssignedID', MIN: 0, MAX: 1},
                {ID: cbc+'ContractedCarrierAssignedID', MIN: 0, MAX: 1},
                {ID: cbc+'PerformingCarrierAssignedID', MIN: 0, MAX: 1},
                {ID: cbc+'SummaryDescription', MIN: 0, MAX: 99999},
                {ID: cbc+'TotalInvoiceAmount', MIN: 0, MAX: 1},
                {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TariffDescription', MIN: 0, MAX: 99999},
                {ID: cbc+'TariffCode', MIN: 0, MAX: 1},
                {ID: cbc+'InsurancePremiumAmount', MIN: 0, MAX: 1},
                {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'LoadingLengthMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'AnimalFoodIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'HumanFoodIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'LivestockIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'BulkCargoIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'ContainerizedIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'GeneralCargoIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'SpecialSecurityIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'ThirdPartyPayerIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CarrierServiceInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'CustomsClearanceServiceInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'ForwarderServiceInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'SpecialServiceInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'ConsolidatableIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'HaulageInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                {ID: cbc+'ChildConsignmentQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'TotalPackagesQuantity', MIN: 0, MAX: 1},
                {ID: cac+'ConsolidatedShipment', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'Consignment', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsigneeAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsignorAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'FreightForwarderAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'BrokerAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractedCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'PerformingCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'SummaryDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalInvoiceAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TariffDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TariffCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InsurancePremiumAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LoadingLengthMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'AnimalFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HumanFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'LivestockIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'BulkCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ContainerizedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'GeneralCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialSecurityIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ThirdPartyPayerIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CarrierServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CustomsClearanceServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ForwarderServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SpecialServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'Information', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsolidatableIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HaulageInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ChildConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalPackagesQuantity', MIN: 0, MAX: 1},
                        {ID: cac+'ConsolidatedShipment', MIN: 0, MAX: 99999},
                        {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'RequestedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                            {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Text', MIN: 0, MAX: 99999},
                                {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 99999},
                            {ID: cac+'Location', MIN: 0, MAX: 1},
                            {ID: cac+'Signature', MIN: 0, MAX: 1},
                            {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        ]},
                        {ID: cac+'RequestedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedPickupTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'Status', MIN: 0, MAX: 99999},
                        {ID: cac+'ChildConsignment', MIN: 0, MAX: 99999},
                        {ID: cac+'ConsigneeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ExporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'ConsignorParty', MIN: 0, MAX: 1},
                        {ID: cac+'ImporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'FreightForwarderParty', MIN: 0, MAX: 1},
                        {ID: cac+'NotifyParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDespatchParty', MIN: 0, MAX: 1},
                        {ID: cac+'FinalDeliveryParty', MIN: 0, MAX: 1},
                        {ID: cac+'PerformingCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'SubstituteCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'LogisticsOperatorParty', MIN: 0, MAX: 1},
                        {ID: cac+'TransportAdvisorParty', MIN: 0, MAX: 1},
                        {ID: cac+'HazardousItemNotificationParty', MIN: 0, MAX: 1},
                        {ID: cac+'InsuranceParty', MIN: 0, MAX: 1},
                        {ID: cac+'MortgageHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'BillOfLadingHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDepartureCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'FinalDestinationCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransitCountry', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransportContract', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'OriginalDespatchTransportationService', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                            {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Priority', MIN: 0, MAX: 1},
                            {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                            {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                                {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                                {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                                {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Condition', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 1, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                                {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                                {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                                {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                                    {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                                    {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    ]},
                                    {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                        {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                        {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                        {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                        {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Reference', MIN: 0, MAX: 1},
                                    {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                ]},
                                {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                            {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                            {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                            {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                            {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                            {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                                {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                                {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                    {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                                {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            ]},
                                            {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                                {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                            {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                                {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                                {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                                {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        ]},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                        {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                        {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                        {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                            {ID: cac+'Item', MIN: 0, MAX: 99999},
                                            {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                            {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                                {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                        {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                            {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                            {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                            {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                            {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                                {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                            {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                            ]},
                                                        ]},
                                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                        {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Package', MIN: 0, MAX: 1},
                                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                        {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                            {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                            {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                                {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                                {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                ]},
                                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'Item', MIN: 1, MAX: 1},
                                                {ID: cac+'Price', MIN: 0, MAX: 1},
                                                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                                {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                                {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                                    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                            {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Despatch', MIN: 0, MAX: 1, LEVEL: sequence(cac+'Despatch')},
                                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        ]},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                        {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                        {ID: cac+'Despatch', MIN: 0, MAX: 1, LEVEL: sequence(cac+'Despatch')},
                                    ]},
                                    {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                                    {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                                    {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Status', MIN: 0, MAX: 99999},
                                    {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                                    {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'Package', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Package', MIN: 0, MAX: 99999},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                                {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                                {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                                {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                                {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                                {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                                {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                            {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                                {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinalDeliveryTransportationService', MIN: 0, MAX: 1},
                        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'CollectPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'DisbursementPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PrepaidPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'ExtraAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'MainCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'PreCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'OnCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                        {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                    {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                    {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                    {ID: cac+'ReturnAddress', MIN: 0, MAX: 1},
                    {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                    {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'ExportCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'RequestedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                    {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                    {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                    {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'Text', MIN: 0, MAX: 99999},
                        {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                            {ID: cbc+'Measure', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 99999},
                    {ID: cac+'Location', MIN: 0, MAX: 1},
                    {ID: cac+'Signature', MIN: 0, MAX: 1},
                    {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'RequestedDeliveryTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                    {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                    {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                    {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'Text', MIN: 0, MAX: 99999},
                        {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                            {ID: cbc+'Measure', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 99999},
                    {ID: cac+'Location', MIN: 0, MAX: 1},
                    {ID: cac+'Signature', MIN: 0, MAX: 1},
                    {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'PlannedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                    {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                    {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                    {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'Text', MIN: 0, MAX: 99999},
                        {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                            {ID: cbc+'Measure', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 99999},
                    {ID: cac+'Location', MIN: 0, MAX: 1},
                    {ID: cac+'Signature', MIN: 0, MAX: 1},
                    {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'PlannedDeliveryTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                    {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                    {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                    {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'Text', MIN: 0, MAX: 99999},
                        {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                            {ID: cbc+'Measure', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 99999},
                    {ID: cac+'Location', MIN: 0, MAX: 1},
                    {ID: cac+'Signature', MIN: 0, MAX: 1},
                    {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'Status', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'Text', MIN: 0, MAX: 99999},
                    {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                        {ID: cbc+'Measure', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ChildConsignment', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CarrierAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsigneeAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsignorAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'FreightForwarderAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'BrokerAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'ContractedCarrierAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'PerformingCarrierAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'SummaryDescription', MIN: 0, MAX: 99999},
                    {ID: cbc+'TotalInvoiceAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TariffDescription', MIN: 0, MAX: 99999},
                    {ID: cbc+'TariffCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InsurancePremiumAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LoadingLengthMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'AnimalFoodIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'HumanFoodIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'LivestockIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'BulkCargoIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ContainerizedIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'GeneralCargoIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialSecurityIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ThirdPartyPayerIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CarrierServiceInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CustomsClearanceServiceInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'ForwarderServiceInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SpecialServiceInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsolidatableIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'HaulageInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'ChildConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalPackagesQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'ConsolidatedShipment', MIN: 0, MAX: 99999},
                    {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'RequestedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                        {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                        {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                        {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                        {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                            {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'Text', MIN: 0, MAX: 99999},
                            {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                            {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 99999},
                        {ID: cac+'Location', MIN: 0, MAX: 1},
                        {ID: cac+'Signature', MIN: 0, MAX: 1},
                        {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    ]},
                    {ID: cac+'RequestedDeliveryTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PlannedPickupTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PlannedDeliveryTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'Status', MIN: 0, MAX: 99999},
                    {ID: cac+'ChildConsignment', MIN: 0, MAX: 99999},
                    {ID: cac+'ConsigneeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ExporterParty', MIN: 0, MAX: 1},
                    {ID: cac+'ConsignorParty', MIN: 0, MAX: 1},
                    {ID: cac+'ImporterParty', MIN: 0, MAX: 1},
                    {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'FreightForwarderParty', MIN: 0, MAX: 1},
                    {ID: cac+'NotifyParty', MIN: 0, MAX: 1},
                    {ID: cac+'OriginalDespatchParty', MIN: 0, MAX: 1},
                    {ID: cac+'FinalDeliveryParty', MIN: 0, MAX: 1},
                    {ID: cac+'PerformingCarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'SubstituteCarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'LogisticsOperatorParty', MIN: 0, MAX: 1},
                    {ID: cac+'TransportAdvisorParty', MIN: 0, MAX: 1},
                    {ID: cac+'HazardousItemNotificationParty', MIN: 0, MAX: 1},
                    {ID: cac+'InsuranceParty', MIN: 0, MAX: 1},
                    {ID: cac+'MortgageHolderParty', MIN: 0, MAX: 1},
                    {ID: cac+'BillOfLadingHolderParty', MIN: 0, MAX: 1},
                    {ID: cac+'OriginalDepartureCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'FinalDestinationCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'TransitCountry', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'TransportContract', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'OriginalDespatchTransportationService', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                        {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Priority', MIN: 0, MAX: 1},
                        {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                        {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                        {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                            {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'Information', MIN: 0, MAX: 99999},
                            {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                            {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                            {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                            {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Condition', MIN: 0, MAX: 1},
                                {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                {ID: cbc+'Measure', MIN: 1, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                            {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                            {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                            {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                            {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                            {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                            {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                            {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                            {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                                {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                                {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                    {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                    {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                    {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                    {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                            ]},
                            {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                {ID: cbc+'Reference', MIN: 0, MAX: 1},
                                {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                                {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            ]},
                            {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                    {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                    {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                    {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                        {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                        {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                        {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                        {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                        {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                            {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                            {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                                {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                        {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                        {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                        {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                        {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                        {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                            {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                            {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                            {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                        ]},
                                        {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                        {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                            {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                                {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                        {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                        {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                            {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                            {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                            {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                            {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                            {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    ]},
                                    {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                    {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                        {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                                        {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                        {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                            {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                    {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                    {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                        {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                        {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                        {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                        {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                            {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                        {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                            {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                            {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                            {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                            {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                            {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                            {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                    {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                    {ID: cac+'Package', MIN: 0, MAX: 1},
                                                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                    {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                        {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                        {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                            {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                            {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                                {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                                {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                                {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                                {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                                {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                            ]},
                                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                            {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                            {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                            {ID: cac+'Item', MIN: 1, MAX: 1},
                                            {ID: cac+'Price', MIN: 0, MAX: 1},
                                            {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                            {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                            {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                        {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                        {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                        {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                            {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                            {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                        {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    ]},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                    {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                    {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                    {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                    {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                    {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Item', MIN: 0, MAX: 99999},
                                    {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                                {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'Status', MIN: 0, MAX: 99999},
                                {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                                {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Package', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'Delivery', MIN: 0, MAX: 1},
                            {ID: cac+'Pickup', MIN: 0, MAX: 1},
                            {ID: cac+'Despatch', MIN: 0, MAX: 1},
                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'Package', MIN: 0, MAX: 99999},
                            {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                        {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                        {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                        {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                        {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                        {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                        {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                            {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                            {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                            {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                            {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                            {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                            {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                            {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                            {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                            {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                            {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                            {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                            {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                            {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                            {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                            {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                        {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                            {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinalDeliveryTransportationService', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                    {ID: cac+'PaymentTerms', MIN: 0, MAX: 1},
                    {ID: cac+'CollectPaymentTerms', MIN: 0, MAX: 1},
                    {ID: cac+'DisbursementPaymentTerms', MIN: 0, MAX: 1},
                    {ID: cac+'PrepaidPaymentTerms', MIN: 0, MAX: 1},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                    {ID: cac+'ExtraAllowanceCharge', MIN: 0, MAX: 99999},
                    {ID: cac+'MainCarriageShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'PreCarriageShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'OnCarriageShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                    {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ConsigneeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ExporterParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ConsignorParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ImporterParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'CarrierParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'FreightForwarderParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'NotifyParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'OriginalDespatchParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'FinalDeliveryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PerformingCarrierParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'SubstituteCarrierParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LogisticsOperatorParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'TransportAdvisorParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'HazardousItemNotificationParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'InsuranceParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MortgageHolderParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'BillOfLadingHolderParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'OriginalDepartureCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                {ID: cac+'FinalDestinationCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                {ID: cac+'TransitCountry', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CountryType')},
                {ID: cac+'TransportContract', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                    {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                    {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                    {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'Text', MIN: 0, MAX: 99999},
                        {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                            {ID: cbc+'Measure', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 99999},
                    {ID: cac+'Location', MIN: 0, MAX: 1},
                    {ID: cac+'Signature', MIN: 0, MAX: 1},
                    {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'OriginalDespatchTransportationService', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                    {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Priority', MIN: 0, MAX: 1},
                    {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                    {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                    {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                        {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'Information', MIN: 0, MAX: 99999},
                        {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                        {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                        {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                        {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                            {ID: cbc+'Measure', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Condition', MIN: 0, MAX: 1},
                            {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                            {ID: cbc+'Measure', MIN: 1, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                        {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                        {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                        {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                        {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                        {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                        {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                        {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                        {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                        {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                            {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                            {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                            ]},
                            {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                        ]},
                        {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Information', MIN: 0, MAX: 99999},
                            {ID: cbc+'Reference', MIN: 0, MAX: 1},
                            {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                            {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                        ]},
                        {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                            {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                            {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                    {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                    {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                    {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                    {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                    {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                        {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                        {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                            {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                    {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                    {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                    {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                    {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                    {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                    {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                        {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                        {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                        {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                        {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                        {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                            {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                        {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                    ]},
                                    {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                        {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                            {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                    {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                    {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                        {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                        {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                        {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                        {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                        {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                        {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                    {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'Item', MIN: 0, MAX: 99999},
                                    {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                    {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                        {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                    {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                    {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                        {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                    {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                        {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                        {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                        {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                {ID: cac+'Package', MIN: 0, MAX: 1},
                                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                    {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                    {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                        {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                            {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                            {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                            {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                            {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                            {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                        ]},
                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                        {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                        {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 1, MAX: 1},
                                        {ID: cac+'Price', MIN: 0, MAX: 1},
                                        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                        {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                        {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                            {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                    {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                    {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                        {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                        {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                    {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                ]},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                {ID: cac+'Despatch', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'Item', MIN: 0, MAX: 99999},
                                {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                            {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                            {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'Status', MIN: 0, MAX: 99999},
                            {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                            {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                            {ID: cac+'Package', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                        {ID: cac+'Pickup', MIN: 0, MAX: 1},
                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                        {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                        {ID: cac+'Package', MIN: 0, MAX: 99999},
                        {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                    {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                    {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                    {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                    {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                    {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                    {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                        {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                        {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                        {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                        {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                        {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                        {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                        {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                        {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                        {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                        {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                    {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                        {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                            {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                            {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinalDeliveryTransportationService', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                    {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Priority', MIN: 0, MAX: 1},
                    {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                    {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                    {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                        {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'Information', MIN: 0, MAX: 99999},
                        {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                        {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                        {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                        {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                            {ID: cbc+'Measure', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Condition', MIN: 0, MAX: 1},
                            {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                            {ID: cbc+'Measure', MIN: 1, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                        {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                        {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                        {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                        {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                        {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                        {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                        {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                        {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                        {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                            {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                            {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                            ]},
                            {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                        ]},
                        {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Information', MIN: 0, MAX: 99999},
                            {ID: cbc+'Reference', MIN: 0, MAX: 1},
                            {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                            {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                        ]},
                        {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                            {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                            {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                    {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                    {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                    {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                    {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                    {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                        {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                        {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                            {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                    {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                    {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                    {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                    {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                    {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                    {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                    {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                        {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                        {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                        {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                        {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                        {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                        {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                        {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                        {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                            {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                        {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                    ]},
                                    {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                                        {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                        {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                            {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                    {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                    {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                    {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                        {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                        {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                        {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                        {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                        {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                        {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                    {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'Item', MIN: 0, MAX: 99999},
                                    {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                    {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                        {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                        {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                        {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                    {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                    {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                        {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                    ]},
                                                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                    {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                        {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                        {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                        {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                        {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                {ID: cac+'Package', MIN: 0, MAX: 1},
                                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                    {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                    {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                        {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                            {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                            {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                            {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                            {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                            {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                        ]},
                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                        {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                        {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 1, MAX: 1},
                                        {ID: cac+'Price', MIN: 0, MAX: 1},
                                        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                        {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                        {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                            {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                        ]},
                                    ]},
                                    {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                    {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                    {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                        {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                        {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                    {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                ]},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                {ID: cac+'Despatch', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'Item', MIN: 0, MAX: 99999},
                                {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                            {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                            {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'Status', MIN: 0, MAX: 99999},
                            {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                            {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                            {ID: cac+'Package', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                        {ID: cac+'Pickup', MIN: 0, MAX: 1},
                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                        {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                        {ID: cac+'Package', MIN: 0, MAX: 99999},
                        {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                    {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                    {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                    {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                    {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                    {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                    {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                        {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                        {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                        {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                        {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                        {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                        {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                        {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                        {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                        {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                        {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                    {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                        {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                            {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                            {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                    ]},
                ]},
                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
                    {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PaymentTerms', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'CollectPaymentTerms', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'DisbursementPaymentTerms', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'PrepaidPaymentTerms', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                    {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                    {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                            {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                            {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                            {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                            {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                        {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                        {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                        {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                        {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                            {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                            {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                            {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                            {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                            {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                            {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                            {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Content', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                            {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                            {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                        ]},
                    ]},
                ]},
                {ID: cac+'ExtraAllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                    {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                    {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                            {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                            {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                            {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                            {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                        {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                        {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                        {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                        {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                            {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                            {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                            {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                            {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                            {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                            {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                            {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Content', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                            {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                            {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                        ]},
                    ]},
                ]},
                {ID: cac+'MainCarriageShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                    {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                    {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                    {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                    {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                    {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                    {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                    {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                    {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                    {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                    {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                    {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PreCarriageShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                    {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                    {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                    {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                    {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                    {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                    {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                    {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                    {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                    {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                    {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                    {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'OnCarriageShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                    {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                    {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                    {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                    {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                    {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                    {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                    {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                    {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                    {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                    {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                    {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                    {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                    {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                        {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                            {ID: cbc+'LineID', MIN: 1, MAX: 1},
                            {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                            {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                            {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                            {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                            {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                            {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                            {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                            {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                            {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                            {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                            {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                            {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                            {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                    {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                    {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                            ]},
                            {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                            {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                            {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                            {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'Signature', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                        ]},
                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                        {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                        {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                        {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                            {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                            {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                            {ID: cac+'Item', MIN: 0, MAX: 99999},
                            {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                        {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                        {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                            {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                            {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                            {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                            {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                        {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                        {ID: cac+'Package', MIN: 0, MAX: 1},
                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                        {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                            {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                ]},
                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                {ID: cac+'Item', MIN: 1, MAX: 1},
                                {ID: cac+'Price', MIN: 0, MAX: 1},
                                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                            {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                            {ID: cac+'Delivery', MIN: 0, MAX: 1},
                            {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Despatch', MIN: 0, MAX: 1},
                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                        ]},
                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                        {ID: cac+'Pickup', MIN: 0, MAX: 1},
                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                        {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                        {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                    {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                    {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                    {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                    {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                    {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'Status', MIN: 0, MAX: 99999},
                    {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                    {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                    {ID: cac+'Package', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
            ]},
            {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                {ID: cac+'Item', MIN: 0, MAX: 99999},
                {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                    {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'LineID', MIN: 1, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                        {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                    {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                            {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                            {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                    {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                    {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                    {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                    {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                    {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                    {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                    {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Date', MIN: 0, MAX: 1},
                                    {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                            {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                            {ID: cac+'Package', MIN: 0, MAX: 1},
                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                    {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                    {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                        {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                        {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                        {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                        {ID: cbc+'Amount', MIN: 0, MAX: 1},
                        {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                        {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    ]},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                    {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                    {ID: cac+'Item', MIN: 1, MAX: 1},
                    {ID: cac+'Price', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                    {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                    {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'Amount', MIN: 1, MAX: 1},
                        {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                    {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                    {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                    {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                    {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                    {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                    {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Despatch', MIN: 0, MAX: 1},
                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
            ]},
            {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'Delivery', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                {ID: cbc+'TrackingID', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryAddress', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'AlternativeDeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'RequestedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PromisedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'EstimatedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryParty', MIN: 0, MAX: 1},
                {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                {ID: cac+'Despatch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'RequestedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RequestedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'GuaranteedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'GuaranteedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                    {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                    {ID: cac+'DespatchAddress', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchLocation', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchParty', MIN: 0, MAX: 1},
                    {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedDespatchPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'RequestedDespatchPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
                    {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MinimumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'BatchQuantity', MIN: 1, MAX: 1},
                    {ID: cbc+'ConsumerUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MaximumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DeliveryUnitType')},
                {ID: cac+'Shipment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'Consignment', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsigneeAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsignorAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'FreightForwarderAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'BrokerAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractedCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'PerformingCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'SummaryDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalInvoiceAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TariffDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TariffCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InsurancePremiumAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LoadingLengthMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'AnimalFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HumanFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'LivestockIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'BulkCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ContainerizedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'GeneralCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialSecurityIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ThirdPartyPayerIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CarrierServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CustomsClearanceServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ForwarderServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SpecialServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'Information', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsolidatableIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HaulageInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ChildConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalPackagesQuantity', MIN: 0, MAX: 1},
                        {ID: cac+'ConsolidatedShipment', MIN: 0, MAX: 99999},
                        {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'RequestedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                            {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Text', MIN: 0, MAX: 99999},
                                {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 99999},
                            {ID: cac+'Location', MIN: 0, MAX: 1},
                            {ID: cac+'Signature', MIN: 0, MAX: 1},
                            {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        ]},
                        {ID: cac+'RequestedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedPickupTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'Status', MIN: 0, MAX: 99999},
                        {ID: cac+'ChildConsignment', MIN: 0, MAX: 99999},
                        {ID: cac+'ConsigneeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ExporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'ConsignorParty', MIN: 0, MAX: 1},
                        {ID: cac+'ImporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'FreightForwarderParty', MIN: 0, MAX: 1},
                        {ID: cac+'NotifyParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDespatchParty', MIN: 0, MAX: 1},
                        {ID: cac+'FinalDeliveryParty', MIN: 0, MAX: 1},
                        {ID: cac+'PerformingCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'SubstituteCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'LogisticsOperatorParty', MIN: 0, MAX: 1},
                        {ID: cac+'TransportAdvisorParty', MIN: 0, MAX: 1},
                        {ID: cac+'HazardousItemNotificationParty', MIN: 0, MAX: 1},
                        {ID: cac+'InsuranceParty', MIN: 0, MAX: 1},
                        {ID: cac+'MortgageHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'BillOfLadingHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDepartureCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'FinalDestinationCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransitCountry', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransportContract', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'OriginalDespatchTransportationService', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                            {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Priority', MIN: 0, MAX: 1},
                            {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                            {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                                {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                                {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                                {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Condition', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 1, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                                {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                                {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                                {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                                    {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                                    {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    ]},
                                    {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                        {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                        {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                        {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                        {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Reference', MIN: 0, MAX: 1},
                                    {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                ]},
                                {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                            {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                            {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                            {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                            {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                            {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                                {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                                {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                    {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                                {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            ]},
                                            {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                                {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                            {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                                {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                                {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                                {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        ]},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                        {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                        {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                        {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                            {ID: cac+'Item', MIN: 0, MAX: 99999},
                                            {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                            {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                                {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                        {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                            {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                            {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                            {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                            {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                                {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                            {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                            ]},
                                                        ]},
                                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                        {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Package', MIN: 0, MAX: 1},
                                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                        {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                            {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                            {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                                {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                                {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                ]},
                                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'Item', MIN: 1, MAX: 1},
                                                {ID: cac+'Price', MIN: 0, MAX: 1},
                                                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                                {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                                {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                                    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                            {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        ]},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                        {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                                    {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                                    {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Status', MIN: 0, MAX: 99999},
                                    {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                                    {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'Package', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Package', MIN: 0, MAX: 99999},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                                {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                                {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                                {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                                {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                                {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                                {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                            {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                                {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinalDeliveryTransportationService', MIN: 0, MAX: 1},
                        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'CollectPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'DisbursementPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PrepaidPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'ExtraAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'MainCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'PreCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'OnCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                        {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                    {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                    {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                    {ID: cac+'ReturnAddress', MIN: 0, MAX: 1},
                    {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                    {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'ExportCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                        {ID: cbc+'LineID', MIN: 1, MAX: 1},
                        {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                        {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                        {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                        {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                        {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                        {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                            {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                            {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                        {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                        {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                        {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                        {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                        {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                        {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                            {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                            {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                            {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                            {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                            {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                            {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                            {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                            {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                            {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                            {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                            {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                        ]},
                        {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                        {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                            {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                            {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                            {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                            {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                        {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                        {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                            {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                            {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                            {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                            {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                            {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                            {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                            {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'Signature', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                    ]},
                    {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                    {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                    {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                    {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                        {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                        {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                        {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                            {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                            {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                            {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                    {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                    {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                        {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                        {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                            {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                            {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                            {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                            {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                            {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Date', MIN: 0, MAX: 1},
                                            {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                    {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                    {ID: cac+'Package', MIN: 0, MAX: 1},
                                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                    {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                        {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                            {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                            {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            ]},
                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                            {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                            {ID: cac+'Item', MIN: 1, MAX: 1},
                            {ID: cac+'Price', MIN: 0, MAX: 1},
                            {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                            {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                            {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                        {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                        {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                        {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                            {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                            {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                            {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                            {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                            {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                            {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                        {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                        {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                        {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                        {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                    ]},
                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                    {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                    {ID: cac+'Pickup', MIN: 0, MAX: 1},
                    {ID: cac+'Despatch', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                    {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                    {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'Item', MIN: 0, MAX: 99999},
                    {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                {ID: cac+'Status', MIN: 0, MAX: 99999},
                {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                {ID: cac+'Package', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'ReturnAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'OriginAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'ExportCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                    {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                    {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                        {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                        {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                        {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                        {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                        {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                        {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Content', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                        {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                        {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                    ]},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1, LEVEL: [
        {ID: cbc+'ID', MIN: 0, MAX: 1},
        {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
        {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
        {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
        {ID: cbc+'Amount', MIN: 0, MAX: 1},
        {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
        ]},
        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
            {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
            {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
            {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
            {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
            {ID: cbc+'Amount', MIN: 1, MAX: 1},
            {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
            {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
            {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
            {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxCategory', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                    {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                    {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                    {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                    {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                    {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                    {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                    {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Content', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                    {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                    {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 0, MAX: 1},
        {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
        {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
        {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
        {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
        {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
        {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
        {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
            {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
            {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
            {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
            {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
            {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
            {ID: cbc+'HolderName', MIN: 0, MAX: 1},
        ]},
        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
        ]},
        {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
        ]},
        {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'AccountID', MIN: 1, MAX: 1},
        ]},
        {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
            {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
            {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
            {ID: cac+'PayerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Content', MIN: 0, MAX: 99999},
            ]},
        ]},
        {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'FinancingParty', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
            {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
        ]},
    ]},
    {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 0, MAX: 1},
        {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
        {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
        {ID: cbc+'Note', MIN: 0, MAX: 99999},
        {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
        {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
        {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
        {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
        {ID: cbc+'Amount', MIN: 0, MAX: 1},
        {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
        {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
        {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
        {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
        {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
        {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
        {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'ExchangeRate', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
            {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
            {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
            {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
            {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
            {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
            {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
            {ID: cbc+'Date', MIN: 0, MAX: 1},
            {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
    ]},
    {ID: cac+'PrepaidPayment', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 0, MAX: 1},
        {ID: cbc+'PaidAmount', MIN: 0, MAX: 1},
        {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
        {ID: cbc+'PaidDate', MIN: 0, MAX: 1},
        {ID: cbc+'PaidTime', MIN: 0, MAX: 1},
        {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
    ]},
    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 0, MAX: 1},
        {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
        {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
        {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
        {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
        {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
        {ID: cbc+'Amount', MIN: 1, MAX: 1},
        {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
        {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
        {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
        {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'Percent', MIN: 0, MAX: 1},
            {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
            {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
            {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
            {ID: cbc+'TierRange', MIN: 0, MAX: 1},
            {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
            {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
            {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
            {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                {ID: cac+'TaxCategory', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
            {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
            {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
            {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
            {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
            {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                {ID: cbc+'HolderName', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
            {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
            {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'AccountID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                {ID: cac+'PayerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Content', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancingParty', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
            ]},
        ]},
    ]},
    {ID: cac+'TaxExchangeRate', MIN: 0, MAX: 1, LEVEL: [
        {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
        {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
        {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
        {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
        {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
        {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
        {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
        {ID: cbc+'Date', MIN: 0, MAX: 1},
        {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                {ID: cbc+'TrackingID', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryAddress', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'AlternativeDeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'RequestedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PromisedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'EstimatedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryParty', MIN: 0, MAX: 1},
                {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                {ID: cac+'Despatch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'RequestedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RequestedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'GuaranteedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'GuaranteedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                    {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                    {ID: cac+'DespatchAddress', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchLocation', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchParty', MIN: 0, MAX: 1},
                    {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedDespatchPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'RequestedDespatchPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
                    {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MinimumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'BatchQuantity', MIN: 1, MAX: 1},
                    {ID: cbc+'ConsumerUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MaximumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DeliveryUnitType')},
                {ID: cac+'Shipment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'Consignment', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsigneeAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsignorAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'FreightForwarderAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'BrokerAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractedCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'PerformingCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'SummaryDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalInvoiceAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TariffDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TariffCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InsurancePremiumAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LoadingLengthMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'AnimalFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HumanFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'LivestockIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'BulkCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ContainerizedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'GeneralCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialSecurityIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ThirdPartyPayerIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CarrierServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CustomsClearanceServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ForwarderServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SpecialServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'Information', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsolidatableIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HaulageInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ChildConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalPackagesQuantity', MIN: 0, MAX: 1},
                        {ID: cac+'ConsolidatedShipment', MIN: 0, MAX: 99999},
                        {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'RequestedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                            {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Text', MIN: 0, MAX: 99999},
                                {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 99999},
                            {ID: cac+'Location', MIN: 0, MAX: 1},
                            {ID: cac+'Signature', MIN: 0, MAX: 1},
                            {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        ]},
                        {ID: cac+'RequestedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedPickupTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'Status', MIN: 0, MAX: 99999},
                        {ID: cac+'ChildConsignment', MIN: 0, MAX: 99999},
                        {ID: cac+'ConsigneeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ExporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'ConsignorParty', MIN: 0, MAX: 1},
                        {ID: cac+'ImporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'FreightForwarderParty', MIN: 0, MAX: 1},
                        {ID: cac+'NotifyParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDespatchParty', MIN: 0, MAX: 1},
                        {ID: cac+'FinalDeliveryParty', MIN: 0, MAX: 1},
                        {ID: cac+'PerformingCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'SubstituteCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'LogisticsOperatorParty', MIN: 0, MAX: 1},
                        {ID: cac+'TransportAdvisorParty', MIN: 0, MAX: 1},
                        {ID: cac+'HazardousItemNotificationParty', MIN: 0, MAX: 1},
                        {ID: cac+'InsuranceParty', MIN: 0, MAX: 1},
                        {ID: cac+'MortgageHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'BillOfLadingHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDepartureCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'FinalDestinationCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransitCountry', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransportContract', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'OriginalDespatchTransportationService', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                            {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Priority', MIN: 0, MAX: 1},
                            {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                            {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                                {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                                {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                                {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Condition', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 1, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                                {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                                {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                                {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                                    {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                                    {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    ]},
                                    {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                        {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                        {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                        {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                        {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Reference', MIN: 0, MAX: 1},
                                    {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                ]},
                                {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                            {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                            {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                            {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                            {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                            {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                                {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                                {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                    {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                                {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            ]},
                                            {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                                {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                            {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                                {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                                {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                                {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        ]},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                        {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                        {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                        {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                            {ID: cac+'Item', MIN: 0, MAX: 99999},
                                            {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                            {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                                {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                        {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                            {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                            {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                            {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                            {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                                {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                            {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                            ]},
                                                        ]},
                                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                        {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Package', MIN: 0, MAX: 1},
                                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                        {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                            {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                            {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                                {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                                {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                ]},
                                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'Item', MIN: 1, MAX: 1},
                                                {ID: cac+'Price', MIN: 0, MAX: 1},
                                                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                                {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                                {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                                    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                            {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        ]},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                        {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                                    {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                                    {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Status', MIN: 0, MAX: 99999},
                                    {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                                    {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'Package', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Package', MIN: 0, MAX: 99999},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                                {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                                {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                                {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                                {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                                {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                                {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                            {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                                {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinalDeliveryTransportationService', MIN: 0, MAX: 1},
                        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'CollectPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'DisbursementPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PrepaidPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'ExtraAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'MainCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'PreCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'OnCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                        {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                    {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                    {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                    {ID: cac+'ReturnAddress', MIN: 0, MAX: 1},
                    {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                    {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'ExportCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
        {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
        {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
        {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
        {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
        {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
        {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
        {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
        {ID: cbc+'Date', MIN: 0, MAX: 1},
        {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                {ID: cbc+'TrackingID', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryAddress', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'AlternativeDeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'RequestedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PromisedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'EstimatedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryParty', MIN: 0, MAX: 1},
                {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                {ID: cac+'Despatch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'RequestedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RequestedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'GuaranteedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'GuaranteedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                    {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                    {ID: cac+'DespatchAddress', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchLocation', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchParty', MIN: 0, MAX: 1},
                    {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedDespatchPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'RequestedDespatchPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
                    {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MinimumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'BatchQuantity', MIN: 1, MAX: 1},
                    {ID: cbc+'ConsumerUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MaximumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DeliveryUnitType')},
                {ID: cac+'Shipment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'Consignment', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsigneeAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsignorAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'FreightForwarderAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'BrokerAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractedCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'PerformingCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'SummaryDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalInvoiceAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TariffDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TariffCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InsurancePremiumAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LoadingLengthMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'AnimalFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HumanFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'LivestockIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'BulkCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ContainerizedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'GeneralCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialSecurityIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ThirdPartyPayerIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CarrierServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CustomsClearanceServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ForwarderServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SpecialServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'Information', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsolidatableIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HaulageInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ChildConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalPackagesQuantity', MIN: 0, MAX: 1},
                        {ID: cac+'ConsolidatedShipment', MIN: 0, MAX: 99999},
                        {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'RequestedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                            {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Text', MIN: 0, MAX: 99999},
                                {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 99999},
                            {ID: cac+'Location', MIN: 0, MAX: 1},
                            {ID: cac+'Signature', MIN: 0, MAX: 1},
                            {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        ]},
                        {ID: cac+'RequestedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedPickupTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'Status', MIN: 0, MAX: 99999},
                        {ID: cac+'ChildConsignment', MIN: 0, MAX: 99999},
                        {ID: cac+'ConsigneeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ExporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'ConsignorParty', MIN: 0, MAX: 1},
                        {ID: cac+'ImporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'FreightForwarderParty', MIN: 0, MAX: 1},
                        {ID: cac+'NotifyParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDespatchParty', MIN: 0, MAX: 1},
                        {ID: cac+'FinalDeliveryParty', MIN: 0, MAX: 1},
                        {ID: cac+'PerformingCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'SubstituteCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'LogisticsOperatorParty', MIN: 0, MAX: 1},
                        {ID: cac+'TransportAdvisorParty', MIN: 0, MAX: 1},
                        {ID: cac+'HazardousItemNotificationParty', MIN: 0, MAX: 1},
                        {ID: cac+'InsuranceParty', MIN: 0, MAX: 1},
                        {ID: cac+'MortgageHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'BillOfLadingHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDepartureCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'FinalDestinationCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransitCountry', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransportContract', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'OriginalDespatchTransportationService', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                            {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Priority', MIN: 0, MAX: 1},
                            {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                            {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                                {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                                {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                                {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Condition', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 1, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                                {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                                {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                                {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                                    {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                                    {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    ]},
                                    {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                        {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                        {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                        {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                        {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Reference', MIN: 0, MAX: 1},
                                    {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                ]},
                                {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                            {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                            {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                            {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                            {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                            {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                                {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                                {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                    {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                                {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            ]},
                                            {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                                {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                            {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                                {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                                {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                                {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        ]},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                        {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                        {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                        {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                            {ID: cac+'Item', MIN: 0, MAX: 99999},
                                            {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                            {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                                {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                        {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                            {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                            {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                            {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                            {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                                {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                            {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                            ]},
                                                        ]},
                                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                        {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Package', MIN: 0, MAX: 1},
                                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                        {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                            {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                            {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                                {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                                {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                ]},
                                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'Item', MIN: 1, MAX: 1},
                                                {ID: cac+'Price', MIN: 0, MAX: 1},
                                                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                                {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                                {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                                    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                            {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        ]},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                        {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                                    {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                                    {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Status', MIN: 0, MAX: 99999},
                                    {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                                    {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'Package', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Package', MIN: 0, MAX: 99999},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                                {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                                {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                                {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                                {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                                {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                                {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                            {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                                {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinalDeliveryTransportationService', MIN: 0, MAX: 1},
                        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'CollectPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'DisbursementPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PrepaidPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'ExtraAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'MainCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'PreCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'OnCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                        {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                    {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                    {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                    {ID: cac+'ReturnAddress', MIN: 0, MAX: 1},
                    {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                    {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'ExportCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'PaymentExchangeRate', MIN: 0, MAX: 1, LEVEL: [
        {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
        {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
        {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
        {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
        {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
        {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
        {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
        {ID: cbc+'Date', MIN: 0, MAX: 1},
        {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                {ID: cbc+'TrackingID', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryAddress', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'AlternativeDeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'RequestedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PromisedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'EstimatedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryParty', MIN: 0, MAX: 1},
                {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                {ID: cac+'Despatch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'RequestedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RequestedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'GuaranteedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'GuaranteedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                    {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                    {ID: cac+'DespatchAddress', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchLocation', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchParty', MIN: 0, MAX: 1},
                    {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedDespatchPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'RequestedDespatchPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
                    {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MinimumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'BatchQuantity', MIN: 1, MAX: 1},
                    {ID: cbc+'ConsumerUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MaximumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DeliveryUnitType')},
                {ID: cac+'Shipment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'Consignment', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsigneeAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsignorAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'FreightForwarderAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'BrokerAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractedCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'PerformingCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'SummaryDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalInvoiceAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TariffDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TariffCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InsurancePremiumAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LoadingLengthMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'AnimalFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HumanFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'LivestockIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'BulkCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ContainerizedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'GeneralCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialSecurityIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ThirdPartyPayerIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CarrierServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CustomsClearanceServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ForwarderServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SpecialServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'Information', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsolidatableIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HaulageInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ChildConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalPackagesQuantity', MIN: 0, MAX: 1},
                        {ID: cac+'ConsolidatedShipment', MIN: 0, MAX: 99999},
                        {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'RequestedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                            {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Text', MIN: 0, MAX: 99999},
                                {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 99999},
                            {ID: cac+'Location', MIN: 0, MAX: 1},
                            {ID: cac+'Signature', MIN: 0, MAX: 1},
                            {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        ]},
                        {ID: cac+'RequestedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedPickupTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'Status', MIN: 0, MAX: 99999},
                        {ID: cac+'ChildConsignment', MIN: 0, MAX: 99999},
                        {ID: cac+'ConsigneeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ExporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'ConsignorParty', MIN: 0, MAX: 1},
                        {ID: cac+'ImporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'FreightForwarderParty', MIN: 0, MAX: 1},
                        {ID: cac+'NotifyParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDespatchParty', MIN: 0, MAX: 1},
                        {ID: cac+'FinalDeliveryParty', MIN: 0, MAX: 1},
                        {ID: cac+'PerformingCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'SubstituteCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'LogisticsOperatorParty', MIN: 0, MAX: 1},
                        {ID: cac+'TransportAdvisorParty', MIN: 0, MAX: 1},
                        {ID: cac+'HazardousItemNotificationParty', MIN: 0, MAX: 1},
                        {ID: cac+'InsuranceParty', MIN: 0, MAX: 1},
                        {ID: cac+'MortgageHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'BillOfLadingHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDepartureCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'FinalDestinationCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransitCountry', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransportContract', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'OriginalDespatchTransportationService', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                            {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Priority', MIN: 0, MAX: 1},
                            {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                            {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                                {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                                {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                                {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Condition', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 1, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                                {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                                {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                                {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                                    {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                                    {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    ]},
                                    {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                        {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                        {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                        {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                        {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Reference', MIN: 0, MAX: 1},
                                    {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                ]},
                                {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                            {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                            {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                            {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                            {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                            {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                                {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                                {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                    {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                                {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            ]},
                                            {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                                {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                            {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                                {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                                {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                                {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        ]},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                        {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                        {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                        {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                            {ID: cac+'Item', MIN: 0, MAX: 99999},
                                            {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                            {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                                {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                        {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                            {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                            {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                            {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                            {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                                {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                            {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                            ]},
                                                        ]},
                                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                        {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Package', MIN: 0, MAX: 1},
                                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                        {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                            {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                            {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                                {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                                {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                ]},
                                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'Item', MIN: 1, MAX: 1},
                                                {ID: cac+'Price', MIN: 0, MAX: 1},
                                                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                                {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                                {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                                    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                            {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        ]},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                        {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                                    {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                                    {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Status', MIN: 0, MAX: 99999},
                                    {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                                    {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'Package', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Package', MIN: 0, MAX: 99999},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                                {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                                {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                                {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                                {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                                {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                                {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                            {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                                {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinalDeliveryTransportationService', MIN: 0, MAX: 1},
                        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'CollectPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'DisbursementPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PrepaidPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'ExtraAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'MainCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'PreCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'OnCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                        {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                    {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                    {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                    {ID: cac+'ReturnAddress', MIN: 0, MAX: 1},
                    {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                    {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'ExportCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'PaymentAlternativeExchangeRate', MIN: 0, MAX: 1, LEVEL: [
        {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
        {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
        {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
        {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
        {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
        {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
        {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
        {ID: cbc+'Date', MIN: 0, MAX: 1},
        {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                {ID: cbc+'TrackingID', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryAddress', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'AlternativeDeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'RequestedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PromisedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'EstimatedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryParty', MIN: 0, MAX: 1},
                {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                {ID: cac+'Despatch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'RequestedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RequestedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'EstimatedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ActualDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'GuaranteedDespatchDate', MIN: 0, MAX: 1},
                    {ID: cbc+'GuaranteedDespatchTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                    {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                    {ID: cac+'DespatchAddress', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchLocation', MIN: 0, MAX: 1},
                    {ID: cac+'DespatchParty', MIN: 0, MAX: 1},
                    {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'EstimatedDespatchPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'RequestedDespatchPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
                    {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MinimumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'BatchQuantity', MIN: 1, MAX: 1},
                    {ID: cbc+'ConsumerUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MaximumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DeliveryUnitType')},
                {ID: cac+'Shipment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'Consignment', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsigneeAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsignorAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'FreightForwarderAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'BrokerAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractedCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'PerformingCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'SummaryDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalInvoiceAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TariffDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TariffCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InsurancePremiumAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LoadingLengthMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'AnimalFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HumanFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'LivestockIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'BulkCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ContainerizedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'GeneralCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialSecurityIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ThirdPartyPayerIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CarrierServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CustomsClearanceServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ForwarderServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SpecialServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'Information', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsolidatableIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HaulageInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ChildConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalPackagesQuantity', MIN: 0, MAX: 1},
                        {ID: cac+'ConsolidatedShipment', MIN: 0, MAX: 99999},
                        {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'RequestedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                            {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Text', MIN: 0, MAX: 99999},
                                {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 99999},
                            {ID: cac+'Location', MIN: 0, MAX: 1},
                            {ID: cac+'Signature', MIN: 0, MAX: 1},
                            {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        ]},
                        {ID: cac+'RequestedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedPickupTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'Status', MIN: 0, MAX: 99999},
                        {ID: cac+'ChildConsignment', MIN: 0, MAX: 99999},
                        {ID: cac+'ConsigneeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ExporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'ConsignorParty', MIN: 0, MAX: 1},
                        {ID: cac+'ImporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'FreightForwarderParty', MIN: 0, MAX: 1},
                        {ID: cac+'NotifyParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDespatchParty', MIN: 0, MAX: 1},
                        {ID: cac+'FinalDeliveryParty', MIN: 0, MAX: 1},
                        {ID: cac+'PerformingCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'SubstituteCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'LogisticsOperatorParty', MIN: 0, MAX: 1},
                        {ID: cac+'TransportAdvisorParty', MIN: 0, MAX: 1},
                        {ID: cac+'HazardousItemNotificationParty', MIN: 0, MAX: 1},
                        {ID: cac+'InsuranceParty', MIN: 0, MAX: 1},
                        {ID: cac+'MortgageHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'BillOfLadingHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDepartureCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'FinalDestinationCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransitCountry', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransportContract', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'OriginalDespatchTransportationService', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                            {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Priority', MIN: 0, MAX: 1},
                            {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                            {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                                {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                                {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                                {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Condition', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 1, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                                {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                                {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                                {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                                    {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                                    {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    ]},
                                    {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                        {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                        {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                        {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                        {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Reference', MIN: 0, MAX: 1},
                                    {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                ]},
                                {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                            {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                            {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                            {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                            {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                            {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                                {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                                {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                    {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                                {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            ]},
                                            {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                                {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                            {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                                {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                                {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                                {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        ]},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                        {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                        {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                        {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                            {ID: cac+'Item', MIN: 0, MAX: 99999},
                                            {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                            {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                                {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                        {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                            {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                            {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                            {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                            {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                                {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                            {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                            ]},
                                                        ]},
                                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                        {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Package', MIN: 0, MAX: 1},
                                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                        {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                            {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                            {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                                {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                                {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                ]},
                                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'Item', MIN: 1, MAX: 1},
                                                {ID: cac+'Price', MIN: 0, MAX: 1},
                                                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                                {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                                {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                                    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                            {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        ]},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                        {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                                    {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                                    {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Status', MIN: 0, MAX: 99999},
                                    {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                                    {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'Package', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Package', MIN: 0, MAX: 99999},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                                {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                                {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                                {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                                {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                                {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                                {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                            {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                                {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinalDeliveryTransportationService', MIN: 0, MAX: 1},
                        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'CollectPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'DisbursementPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PrepaidPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'ExtraAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'MainCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'PreCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'OnCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                        {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                    {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                    {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                    {ID: cac+'ReturnAddress', MIN: 0, MAX: 1},
                    {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                    {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'ExportCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
        {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
        {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
        {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
            {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
            {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
            {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
            {ID: cbc+'Percent', MIN: 0, MAX: 1},
            {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
            {ID: cbc+'TierRange', MIN: 0, MAX: 1},
            {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
            {ID: cac+'TaxCategory', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
        {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
        {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
        {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
        {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
            {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
            {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
            {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
            {ID: cbc+'Percent', MIN: 0, MAX: 1},
            {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
            {ID: cbc+'TierRange', MIN: 0, MAX: 1},
            {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
            {ID: cac+'TaxCategory', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
        ]},
    ]},
    {ID: cac+'LegalMonetaryTotal', MIN: 1, MAX: 1, LEVEL: [
        {ID: cbc+'LineExtensionAmount', MIN: 0, MAX: 1},
        {ID: cbc+'TaxExclusiveAmount', MIN: 0, MAX: 1},
        {ID: cbc+'TaxInclusiveAmount', MIN: 0, MAX: 1},
        {ID: cbc+'AllowanceTotalAmount', MIN: 0, MAX: 1},
        {ID: cbc+'ChargeTotalAmount', MIN: 0, MAX: 1},
        {ID: cbc+'PrepaidAmount', MIN: 0, MAX: 1},
        {ID: cbc+'PayableRoundingAmount', MIN: 0, MAX: 1},
        {ID: cbc+'PayableAmount', MIN: 1, MAX: 1},
        {ID: cbc+'PayableAlternativeAmount', MIN: 0, MAX: 1},
    ]},
    {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
        {ID: cbc+'ID', MIN: 1, MAX: 1},
        {ID: cbc+'UUID', MIN: 0, MAX: 1},
        {ID: cbc+'Note', MIN: 0, MAX: 99999},
        {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
        {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
        {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
        {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
        {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
        {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
        {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
        {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
        {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'LineID', MIN: 1, MAX: 1},
            {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
            {ID: cac+'OrderReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'SalesOrderID', MIN: 0, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'CustomerReference', MIN: 0, MAX: 1},
                {ID: cbc+'OrderTypeCode', MIN: 0, MAX: 1},
                {ID: cac+'DocumentReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'LineID', MIN: 1, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
            {ID: cac+'DocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'LineID', MIN: 1, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
            {ID: cac+'DocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'BillingReference', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cac+'InvoiceDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'SelfBilledInvoiceDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'CreditNoteDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'SelfBilledCreditNoteDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'DebitNoteDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'ReminderDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'AdditionalDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'BillingReferenceLine', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'Amount', MIN: 0, MAX: 1},
                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                    {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                    {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                            {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                            {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                            {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                            {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                        {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                        {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                        {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                        {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                            {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                            {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                            {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                            {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                            {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                            {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                            {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Content', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                            {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                            {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                        ]},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
            {ID: cbc+'XPath', MIN: 0, MAX: 99999},
            {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
            {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
            {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'URI', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                    {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                    {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                    {ID: cbc+'FileName', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                        {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                        {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                        {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                            {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'URI', MIN: 0, MAX: 1},
                                {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                                {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FileName', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                                {ID: cbc+'Room', MIN: 0, MAX: 1},
                                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Department', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Region', MIN: 0, MAX: 1},
                                {ID: cbc+'District', MIN: 0, MAX: 1},
                                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                            ]},
                            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                    {ID: cac+'Party', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                                {ID: cbc+'Title', MIN: 0, MAX: 1},
                                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                                {ID: cac+'Contact', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cac+'Address', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                ]},
                                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                                {ID: cac+'Party', MIN: 1, MAX: 1},
                                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                            {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
            ]},
            {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                {ID: cac+'SignatoryParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
            {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                    {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                    {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                        {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                        {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                        {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                        {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                        {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Date', MIN: 0, MAX: 1},
                        {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Package', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                    {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                    {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                    {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                        {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                        {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                        {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                            {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                            {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                            {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                    {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                    {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                        {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                        {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                        {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                        {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                            {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                            {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                            {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                            {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                            {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Date', MIN: 0, MAX: 1},
                                            {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                        ]},
                                    ]},
                                    {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                    {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                    {ID: cac+'Package', MIN: 0, MAX: 1},
                                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                    {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                        {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                    ]},
                                ]},
                                {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                            {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                            {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            ]},
                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                            {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                            {ID: cac+'Item', MIN: 1, MAX: 1},
                            {ID: cac+'Price', MIN: 0, MAX: 1},
                            {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                            {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                            {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                        {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                        {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                        {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                            {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                            {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                            {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                            {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                            {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                            {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                        {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                        {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                        {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                        {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                    ]},
                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                    {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                    {ID: cac+'Pickup', MIN: 0, MAX: 1},
                    {ID: cac+'Despatch', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                    {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                    {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                            {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                            {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                            {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                            {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                        {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                        {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                        {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                        {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                            {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                            {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                            {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                            {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                            {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                            {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                            {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                            {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Content', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                            {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                            {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                        ]},
                    ]},
                ]},
                {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                    {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                    {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                    {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                    {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                    {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Date', MIN: 0, MAX: 1},
                    {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'OriginatorParty', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
            {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
            {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
            {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'Name', MIN: 1, MAX: 1},
            ]},
            {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                    {ID: cac+'Party', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Channel', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                {ID: cbc+'Title', MIN: 0, MAX: 1},
                {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                    {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                    {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                    {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                        {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'URI', MIN: 0, MAX: 1},
                            {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                            {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FileName', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                        {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                        {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                        {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                            {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                        ]},
                        {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                        ]},
                        {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                            {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                                {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                            {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Party', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                            {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                            {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Channel', MIN: 0, MAX: 1},
                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                            {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                            {ID: cbc+'Title', MIN: 0, MAX: 1},
                            {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                            {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                            {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                            {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                            {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                            {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                            {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                            {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                            {ID: cac+'Contact', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cac+'Address', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                            ]},
                            {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                        {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                            {ID: cac+'Party', MIN: 1, MAX: 1},
                            {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                            {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                            {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                            {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                        {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'AgentParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                {ID: cac+'Party', MIN: 1, MAX: 1},
                {ID: cac+'SellerContact', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            ]},
        ]},
        {ID: cac+'Delivery', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
            {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
            {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
            {ID: cbc+'ActualDeliveryDate', MIN: 0, MAX: 1},
            {ID: cbc+'ActualDeliveryTime', MIN: 0, MAX: 1},
            {ID: cbc+'LatestDeliveryDate', MIN: 0, MAX: 1},
            {ID: cbc+'LatestDeliveryTime', MIN: 0, MAX: 1},
            {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
            {ID: cbc+'TrackingID', MIN: 0, MAX: 1},
            {ID: cac+'DeliveryAddress', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'AlternativeDeliveryLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'RequestedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'PromisedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'EstimatedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'CarrierParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'DeliveryParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'NotifyParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'Despatch', MIN: 0, MAX: 1, LEVEL: sequence(cac+'Despatch')},
            {ID: cac+'DeliveryTerms', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
                {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
                {ID: cbc+'Amount', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'MinimumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'BatchQuantity', MIN: 1, MAX: 1},
                {ID: cbc+'ConsumerUnitQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'MaximumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DeliveryUnitType')},
            {ID: cac+'Shipment', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                {ID: cac+'Consignment', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CarrierAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsigneeAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsignorAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'FreightForwarderAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'BrokerAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'ContractedCarrierAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'PerformingCarrierAssignedID', MIN: 0, MAX: 1},
                    {ID: cbc+'SummaryDescription', MIN: 0, MAX: 99999},
                    {ID: cbc+'TotalInvoiceAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TariffDescription', MIN: 0, MAX: 99999},
                    {ID: cbc+'TariffCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InsurancePremiumAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LoadingLengthMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'AnimalFoodIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'HumanFoodIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'LivestockIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'BulkCargoIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ContainerizedIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'GeneralCargoIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialSecurityIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ThirdPartyPayerIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CarrierServiceInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CustomsClearanceServiceInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'ForwarderServiceInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SpecialServiceInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsolidatableIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'HaulageInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'ChildConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalPackagesQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'ConsolidatedShipment', MIN: 0, MAX: 99999},
                    {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'RequestedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                        {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                        {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                        {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                        {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                            {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                            {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'Text', MIN: 0, MAX: 99999},
                            {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                            {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'Contact', MIN: 0, MAX: 99999},
                        {ID: cac+'Location', MIN: 0, MAX: 1},
                        {ID: cac+'Signature', MIN: 0, MAX: 1},
                        {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    ]},
                    {ID: cac+'RequestedDeliveryTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PlannedPickupTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'PlannedDeliveryTransportEvent', MIN: 0, MAX: 1},
                    {ID: cac+'Status', MIN: 0, MAX: 99999},
                    {ID: cac+'ChildConsignment', MIN: 0, MAX: 99999},
                    {ID: cac+'ConsigneeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ExporterParty', MIN: 0, MAX: 1},
                    {ID: cac+'ConsignorParty', MIN: 0, MAX: 1},
                    {ID: cac+'ImporterParty', MIN: 0, MAX: 1},
                    {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'FreightForwarderParty', MIN: 0, MAX: 1},
                    {ID: cac+'NotifyParty', MIN: 0, MAX: 1},
                    {ID: cac+'OriginalDespatchParty', MIN: 0, MAX: 1},
                    {ID: cac+'FinalDeliveryParty', MIN: 0, MAX: 1},
                    {ID: cac+'PerformingCarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'SubstituteCarrierParty', MIN: 0, MAX: 1},
                    {ID: cac+'LogisticsOperatorParty', MIN: 0, MAX: 1},
                    {ID: cac+'TransportAdvisorParty', MIN: 0, MAX: 1},
                    {ID: cac+'HazardousItemNotificationParty', MIN: 0, MAX: 1},
                    {ID: cac+'InsuranceParty', MIN: 0, MAX: 1},
                    {ID: cac+'MortgageHolderParty', MIN: 0, MAX: 1},
                    {ID: cac+'BillOfLadingHolderParty', MIN: 0, MAX: 1},
                    {ID: cac+'OriginalDepartureCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'FinalDestinationCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'TransitCountry', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'TransportContract', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                    {ID: cac+'OriginalDespatchTransportationService', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                        {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Priority', MIN: 0, MAX: 1},
                        {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                        {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                        {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                            {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'Information', MIN: 0, MAX: 99999},
                            {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                            {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                            {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                            {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Condition', MIN: 0, MAX: 1},
                                {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                {ID: cbc+'Measure', MIN: 1, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                            {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                            {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                            {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                            {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                            {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                            {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                            {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                            {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                                {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                                {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                                {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                                ]},
                                {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                    {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                    {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                    {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                    {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                    {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                    {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                            ]},
                            {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                {ID: cbc+'Reference', MIN: 0, MAX: 1},
                                {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                                {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                            ]},
                            {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                                {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                    {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                    {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                    {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                        {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                        {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                        {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                        {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                        {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                        {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                            {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                            {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                                {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                        {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                        {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                        {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                        {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                        {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                        {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                            {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                            {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                            {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                            {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                            {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                            {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                        ]},
                                        {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                        {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 1, MAX: 1},
                                            {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                            {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                                {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                            ]},
                                        ]},
                                        {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                        {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                        {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                        {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                            {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                            {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                            {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                            {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                            {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    ]},
                                    {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                    {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                        {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                                        {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                        ]},
                                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                        {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                            {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                            {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                            {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                            {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                            {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                    {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                    {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                    {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                        {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                        {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                        {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                        {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                        {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                            {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                        ]},
                                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                        {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                            {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                            {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                            {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                            {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                            {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                            {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                            {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                    {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                    {ID: cac+'Package', MIN: 0, MAX: 1},
                                                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                    {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                        {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                        {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                    ]},
                                                ]},
                                                {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                            {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                            {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                                {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                                {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                                {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                                {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                                {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                            ]},
                                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                            {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                            {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                            {ID: cac+'Item', MIN: 1, MAX: 1},
                                            {ID: cac+'Price', MIN: 0, MAX: 1},
                                            {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                            {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                            {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                            ]},
                                        ]},
                                        {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                        {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                        {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                        {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                            {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                            {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                            {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                        {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    ]},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                    {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                    {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                    {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                    {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                    {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Item', MIN: 0, MAX: 99999},
                                    {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                                {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'Status', MIN: 0, MAX: 99999},
                                {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                                {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Package', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'Delivery', MIN: 0, MAX: 1},
                            {ID: cac+'Pickup', MIN: 0, MAX: 1},
                            {ID: cac+'Despatch', MIN: 0, MAX: 1},
                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'Package', MIN: 0, MAX: 99999},
                            {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                        {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                        {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                        {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                        {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                        {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                        {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                            {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                            {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                            {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                            {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                            {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                            {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                            {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                            {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                            {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                            {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                            {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                            {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                            {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                            {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                            {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                            {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                            {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                            {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                            {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                            {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                            {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                        {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                            {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'FinalDeliveryTransportationService', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                    {ID: cac+'PaymentTerms', MIN: 0, MAX: 1},
                    {ID: cac+'CollectPaymentTerms', MIN: 0, MAX: 1},
                    {ID: cac+'DisbursementPaymentTerms', MIN: 0, MAX: 1},
                    {ID: cac+'PrepaidPaymentTerms', MIN: 0, MAX: 1},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                    {ID: cac+'ExtraAllowanceCharge', MIN: 0, MAX: 99999},
                    {ID: cac+'MainCarriageShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'PreCarriageShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'OnCarriageShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                    {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999},
                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                {ID: cac+'ReturnAddress', MIN: 0, MAX: 1},
                {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                {ID: cac+'ExportCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
            ]},
        ]},
        {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
            {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
            {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
            {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
            {ID: cbc+'Amount', MIN: 0, MAX: 1},
            {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
            {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
            {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
            {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
            {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'ExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                {ID: cbc+'Date', MIN: 0, MAX: 1},
                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
        ]},
        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
            {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
            {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
            {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
            {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
            {ID: cbc+'Amount', MIN: 1, MAX: 1},
            {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
            {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
            {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
            {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxCategory', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                    ]},
                ]},
            ]},
            {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                    {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                    {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                    {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                    {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                    {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                ]},
                {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                    {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                    {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Content', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                    {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                    {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                ]},
            ]},
        ]},
        {ID: cac+'TaxTotal', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
            {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
            {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
            {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                {ID: cac+'TaxCategory', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
            {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
            {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
            {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                {ID: cac+'TaxCategory', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                ]},
            ]},
        ]},
        {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
            {ID: cbc+'Description', MIN: 0, MAX: 99999},
            {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
            {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
            {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'Name', MIN: 0, MAX: 1},
            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
            {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
            {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
            {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
            {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                    {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                    {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
            ]},
            {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                    {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                ]},
            ]},
            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 1, MAX: 1},
                {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                {ID: cbc+'Value', MIN: 0, MAX: 1},
                {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                    {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                            {ID: cbc+'Floor', MIN: 0, MAX: 1},
                            {ID: cbc+'Room', MIN: 0, MAX: 1},
                            {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                            {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                            {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                            {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                            {ID: cbc+'Department', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                            {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                            {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                            {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                            {ID: cbc+'CityName', MIN: 0, MAX: 1},
                            {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                            {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Region', MIN: 0, MAX: 1},
                            {ID: cbc+'District', MIN: 0, MAX: 1},
                            {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                            {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'Line', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                            ]},
                        ]},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'OriginAddress', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                {ID: cbc+'Floor', MIN: 0, MAX: 1},
                {ID: cbc+'Room', MIN: 0, MAX: 1},
                {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                {ID: cbc+'Department', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                {ID: cbc+'CityName', MIN: 0, MAX: 1},
                {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'Region', MIN: 0, MAX: 1},
                {ID: cbc+'District', MIN: 0, MAX: 1},
                {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Line', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                {ID: cac+'Signature', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
        ]},
        {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
            {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
            {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
            {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
            {ID: cbc+'PriceType', MIN: 0, MAX: 1},
            {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                    {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                    {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                        {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                        {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                        {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                        {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                        {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                        {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Content', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                        {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                        {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                    ]},
                ]},
            ]},
            {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                {ID: cbc+'Date', MIN: 0, MAX: 1},
                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
            ]},
        ]},
        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'ID', MIN: 0, MAX: 1},
            {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
            {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
            {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
            {ID: cbc+'Amount', MIN: 0, MAX: 1},
            {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'Address', MIN: 0, MAX: 1},
                {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
            ]},
            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                    {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                    {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                        {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                        {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                        {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                        {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                        {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                        {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Content', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                        {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                        {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                    ]},
                ]},
            ]},
        ]},
        {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
            {ID: cbc+'ID', MIN: 1, MAX: 1},
            {ID: cbc+'UUID', MIN: 0, MAX: 1},
            {ID: cbc+'Note', MIN: 0, MAX: 99999},
            {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
            {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
            {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
            {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
            {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
            {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
            {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
            {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
            {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'LineID', MIN: 1, MAX: 1},
                {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                {ID: cac+'OrderReference', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'LineID', MIN: 1, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'LineID', MIN: 1, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'BillingReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cac+'InvoiceDocumentReference', MIN: 0, MAX: 1},
                {ID: cac+'SelfBilledInvoiceDocumentReference', MIN: 0, MAX: 1},
                {ID: cac+'CreditNoteDocumentReference', MIN: 0, MAX: 1},
                {ID: cac+'SelfBilledCreditNoteDocumentReference', MIN: 0, MAX: 1},
                {ID: cac+'DebitNoteDocumentReference', MIN: 0, MAX: 1},
                {ID: cac+'ReminderDocumentReference', MIN: 0, MAX: 1},
                {ID: cac+'AdditionalDocumentReference', MIN: 0, MAX: 1},
                {ID: cac+'BillingReferenceLine', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                        {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'Amount', MIN: 1, MAX: 1},
                        {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                            {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                            {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                            {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                            {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                            {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                                {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                                {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                                {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                                {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                            {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                            {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                            {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                            {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                                {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                                {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                                {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                                {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                                {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                                {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                            {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1},
                            {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                            ]},
                            {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                                {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                                {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                                {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                                {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Content', MIN: 0, MAX: 99999},
                                ]},
                            ]},
                            {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                                {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                                {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                                {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                            ]},
                        ]},
                    ]},
                ]},
            ]},
            {ID: cac+'DocumentReference', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'CopyIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentType', MIN: 0, MAX: 1},
                {ID: cbc+'XPath', MIN: 0, MAX: 99999},
                {ID: cbc+'LanguageID', MIN: 0, MAX: 1},
                {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentStatusCode', MIN: 0, MAX: 1},
                {ID: cbc+'DocumentDescription', MIN: 0, MAX: 99999},
                {ID: cac+'Attachment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'EmbeddedDocumentBinaryObject', MIN: 0, MAX: 1},
                    {ID: cac+'ExternalReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'URI', MIN: 0, MAX: 1},
                        {ID: cbc+'DocumentHash', MIN: 0, MAX: 1},
                        {ID: cbc+'HashAlgorithmMethod', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryTime', MIN: 0, MAX: 1},
                        {ID: cbc+'MimeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'EncodingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CharacterSetCode', MIN: 0, MAX: 1},
                        {ID: cbc+'FileName', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'IssuerParty', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                    {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                    {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                    {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Name', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                        {ID: cbc+'Floor', MIN: 0, MAX: 1},
                        {ID: cbc+'Room', MIN: 0, MAX: 1},
                        {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                        {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                        {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                        {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Department', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                        {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                        {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                        {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                        {ID: cbc+'CityName', MIN: 0, MAX: 1},
                        {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Region', MIN: 0, MAX: 1},
                        {ID: cbc+'District', MIN: 0, MAX: 1},
                        {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                        {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'Line', MIN: 1, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                            {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                        {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Address', MIN: 0, MAX: 1},
                        {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                        {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                    ]},
                    {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                        {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                        {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                            {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                        ]},
                        {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                            {ID: cac+'Party', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                        {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                        {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                        {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Channel', MIN: 0, MAX: 1},
                            {ID: cbc+'Value', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                        {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                        {ID: cbc+'Title', MIN: 0, MAX: 1},
                        {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                        {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                        {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                        {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                        {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                        {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                        {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                        {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                        {ID: cac+'Contact', MIN: 0, MAX: 1},
                        {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                            {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                            {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                    {ID: cac+'Address', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        ]},
                        {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                    {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                        {ID: cac+'Party', MIN: 1, MAX: 1},
                        {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                        {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                        {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                        {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                        {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ResultOfVerification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ValidatorID', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationResultCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidationTime', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateProcess', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateTool', MIN: 0, MAX: 1},
                    {ID: cbc+'ValidateToolVersion', MIN: 0, MAX: 1},
                    {ID: cac+'SignatoryParty', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                    {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                    {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                        {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                        {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                            {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                            {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                            {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                            {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                            {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                            {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Date', MIN: 0, MAX: 1},
                            {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                    {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                    {ID: cac+'Package', MIN: 0, MAX: 1},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                    {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                        {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
            ]},
            {ID: cac+'OriginatorParty', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'MarkCareIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'MarkAttentionIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'WebsiteURI', MIN: 0, MAX: 1},
                {ID: cbc+'LogoReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'EndpointID', MIN: 0, MAX: 1},
                {ID: cbc+'IndustryClassificationCode', MIN: 0, MAX: 1},
                {ID: cac+'PartyIdentification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'PartyName', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'Language', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'LocaleCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PostalAddress', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'AddressFormatCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Postbox', MIN: 0, MAX: 1},
                    {ID: cbc+'Floor', MIN: 0, MAX: 1},
                    {ID: cbc+'Room', MIN: 0, MAX: 1},
                    {ID: cbc+'StreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalStreetName', MIN: 0, MAX: 1},
                    {ID: cbc+'BlockName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingName', MIN: 0, MAX: 1},
                    {ID: cbc+'BuildingNumber', MIN: 0, MAX: 1},
                    {ID: cbc+'InhouseMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Department', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkAttention', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkCare', MIN: 0, MAX: 1},
                    {ID: cbc+'PlotIdentification', MIN: 0, MAX: 1},
                    {ID: cbc+'CitySubdivisionName', MIN: 0, MAX: 1},
                    {ID: cbc+'CityName', MIN: 0, MAX: 1},
                    {ID: cbc+'PostalZone', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Region', MIN: 0, MAX: 1},
                    {ID: cbc+'District', MIN: 0, MAX: 1},
                    {ID: cbc+'TimezoneOffset', MIN: 0, MAX: 1},
                    {ID: cac+'AddressLine', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'Line', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'IdentificationCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'CoordinateSystemCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LatitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDegreesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeMinutesMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LongitudeDirectionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AltitudeMeasure', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PhysicalLocation', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cbc+'Conditions', MIN: 0, MAX: 99999},
                    {ID: cbc+'CountrySubentity', MIN: 0, MAX: 1},
                    {ID: cbc+'CountrySubentityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LocationTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InformationURI', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'Address', MIN: 0, MAX: 1},
                    {ID: cac+'SubsidiaryLocation', MIN: 0, MAX: 99999},
                    {ID: cac+'LocationCoordinate', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'LocationCoordinateType')},
                ]},
                {ID: cac+'PartyTaxScheme', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'PartyLegalEntity', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'RegistrationName', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyID', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationExpirationDate', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalFormCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLegalForm', MIN: 0, MAX: 1},
                    {ID: cbc+'SoleProprietorshipIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'CompanyLiquidationStatusCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CorporateStockAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FullyPaidSharesIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'RegistrationAddress', MIN: 0, MAX: 1},
                    {ID: cac+'CorporateRegistrationScheme', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'CorporateRegistrationTypeCode', MIN: 0, MAX: 1},
                        {ID: cac+'JurisdictionRegionAddress', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HeadOfficeParty', MIN: 0, MAX: 1},
                    {ID: cac+'ShareholderParty', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'PartecipationPercent', MIN: 0, MAX: 1},
                        {ID: cac+'Party', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Contact', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Telephone', MIN: 0, MAX: 1},
                    {ID: cbc+'Telefax', MIN: 0, MAX: 1},
                    {ID: cbc+'ElectronicMail', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cac+'OtherCommunication', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ChannelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Channel', MIN: 0, MAX: 1},
                        {ID: cbc+'Value', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'Person', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'FirstName', MIN: 0, MAX: 1},
                    {ID: cbc+'FamilyName', MIN: 0, MAX: 1},
                    {ID: cbc+'Title', MIN: 0, MAX: 1},
                    {ID: cbc+'MiddleName', MIN: 0, MAX: 1},
                    {ID: cbc+'OtherName', MIN: 0, MAX: 1},
                    {ID: cbc+'NameSuffix', MIN: 0, MAX: 1},
                    {ID: cbc+'JobTitle', MIN: 0, MAX: 1},
                    {ID: cbc+'NationalityID', MIN: 0, MAX: 1},
                    {ID: cbc+'GenderCode', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthDate', MIN: 0, MAX: 1},
                    {ID: cbc+'BirthplaceName', MIN: 0, MAX: 1},
                    {ID: cbc+'OrganizationDepartment', MIN: 0, MAX: 1},
                    {ID: cac+'Contact', MIN: 0, MAX: 1},
                    {ID: cac+'FinancialAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'AliasName', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'AccountFormatCode', MIN: 0, MAX: 1},
                        {ID: cbc+'CurrencyCode', MIN: 0, MAX: 1},
                        {ID: cbc+'PaymentNote', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancialInstitutionBranch', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cac+'FinancialInstitution', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'Name', MIN: 0, MAX: 1},
                                {ID: cac+'Address', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'Address', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'Country', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    ]},
                    {ID: cac+'IdentityDocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'ResidenceAddress', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AgentParty', MIN: 0, MAX: 1},
                {ID: cac+'ServiceProviderParty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ServiceType', MIN: 0, MAX: 99999},
                    {ID: cac+'Party', MIN: 1, MAX: 1},
                    {ID: cac+'SellerContact', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'PowerOfAttorney', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'NotaryParty', MIN: 0, MAX: 1},
                    {ID: cac+'AgentParty', MIN: 1, MAX: 1},
                    {ID: cac+'WitnessParty', MIN: 0, MAX: 99999},
                    {ID: cac+'MandateDocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'FinancialAccount', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'Delivery', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'ActualDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryDate', MIN: 0, MAX: 1},
                {ID: cbc+'LatestDeliveryTime', MIN: 0, MAX: 1},
                {ID: cbc+'ReleaseID', MIN: 0, MAX: 1},
                {ID: cbc+'TrackingID', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryAddress', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'AlternativeDeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'RequestedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PromisedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'EstimatedDeliveryPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryParty', MIN: 0, MAX: 1},
                {ID: cac+'NotifyParty', MIN: 0, MAX: 99999},
                {ID: cac+'Despatch', MIN: 0, MAX: 1, LEVEL: sequence(cac+'Despatch')},
                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
                    {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                    {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MinimumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'BatchQuantity', MIN: 1, MAX: 1},
                    {ID: cbc+'ConsumerUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'MaximumDeliveryUnit', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DeliveryUnitType')},
                {ID: cac+'Shipment', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                    {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                    {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                    {ID: cac+'Consignment', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'CarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsigneeAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsignorAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'FreightForwarderAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'BrokerAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'ContractedCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'PerformingCarrierAssignedID', MIN: 0, MAX: 1},
                        {ID: cbc+'SummaryDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalInvoiceAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TariffDescription', MIN: 0, MAX: 99999},
                        {ID: cbc+'TariffCode', MIN: 0, MAX: 1},
                        {ID: cbc+'InsurancePremiumAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'LoadingLengthMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'AnimalFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HumanFoodIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'LivestockIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'BulkCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ContainerizedIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'GeneralCargoIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialSecurityIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'ThirdPartyPayerIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'CarrierServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'CustomsClearanceServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ForwarderServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SpecialServiceInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ShippingPriorityLevelCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                        {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'Information', MIN: 0, MAX: 99999},
                        {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalTransportHandlingUnitQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SpecialInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'SplitConsignmentIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'DeliveryInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'ConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'ConsolidatableIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'HaulageInstructions', MIN: 0, MAX: 99999},
                        {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                        {ID: cbc+'ChildConsignmentQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'TotalPackagesQuantity', MIN: 0, MAX: 1},
                        {ID: cac+'ConsolidatedShipment', MIN: 0, MAX: 99999},
                        {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 1, MAX: 1},
                            {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'RequestedPickupTransportEvent', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'IdentificationID', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceDate', MIN: 0, MAX: 1},
                            {ID: cbc+'OccurrenceTime', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportEventTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cbc+'CompletionIndicator', MIN: 0, MAX: 1},
                            {ID: cac+'ReportedShipment', MIN: 0, MAX: 1},
                            {ID: cac+'CurrentStatus', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ConditionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceDate', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferenceTime', MIN: 0, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'StatusReasonCode', MIN: 0, MAX: 1},
                                {ID: cbc+'StatusReason', MIN: 0, MAX: 99999},
                                {ID: cbc+'SequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Text', MIN: 0, MAX: 99999},
                                {ID: cbc+'IndicationIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                {ID: cbc+'ReliabilityPercent', MIN: 0, MAX: 1},
                                {ID: cac+'Condition', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'Contact', MIN: 0, MAX: 99999},
                            {ID: cac+'Location', MIN: 0, MAX: 1},
                            {ID: cac+'Signature', MIN: 0, MAX: 1},
                            {ID: cac+'Period', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                        ]},
                        {ID: cac+'RequestedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedPickupTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'PlannedDeliveryTransportEvent', MIN: 0, MAX: 1},
                        {ID: cac+'Status', MIN: 0, MAX: 99999},
                        {ID: cac+'ChildConsignment', MIN: 0, MAX: 99999},
                        {ID: cac+'ConsigneeParty', MIN: 0, MAX: 1},
                        {ID: cac+'ExporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'ConsignorParty', MIN: 0, MAX: 1},
                        {ID: cac+'ImporterParty', MIN: 0, MAX: 1},
                        {ID: cac+'CarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'FreightForwarderParty', MIN: 0, MAX: 1},
                        {ID: cac+'NotifyParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDespatchParty', MIN: 0, MAX: 1},
                        {ID: cac+'FinalDeliveryParty', MIN: 0, MAX: 1},
                        {ID: cac+'PerformingCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'SubstituteCarrierParty', MIN: 0, MAX: 1},
                        {ID: cac+'LogisticsOperatorParty', MIN: 0, MAX: 1},
                        {ID: cac+'TransportAdvisorParty', MIN: 0, MAX: 1},
                        {ID: cac+'HazardousItemNotificationParty', MIN: 0, MAX: 1},
                        {ID: cac+'InsuranceParty', MIN: 0, MAX: 1},
                        {ID: cac+'MortgageHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'BillOfLadingHolderParty', MIN: 0, MAX: 1},
                        {ID: cac+'OriginalDepartureCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'FinalDestinationCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransitCountry', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CountryType')},
                        {ID: cac+'TransportContract', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueDate', MIN: 0, MAX: 1},
                            {ID: cbc+'IssueTime', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'ContractType', MIN: 0, MAX: 1},
                            {ID: cbc+'Note', MIN: 0, MAX: 99999},
                            {ID: cbc+'VersionID', MIN: 0, MAX: 1},
                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 99999},
                            {ID: cac+'NominationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ContractualDelivery', MIN: 0, MAX: 1},
                        ]},
                        {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                        {ID: cac+'OriginalDespatchTransportationService', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'TransportServiceCode', MIN: 1, MAX: 1},
                            {ID: cbc+'TariffClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'Priority', MIN: 0, MAX: 1},
                            {ID: cbc+'FreightRateClassCode', MIN: 0, MAX: 1},
                            {ID: cbc+'TransportationServiceDescription', MIN: 0, MAX: 99999},
                            {ID: cbc+'TransportationServiceDetailsURI', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationDate', MIN: 0, MAX: 1},
                            {ID: cbc+'NominationTime', MIN: 0, MAX: 1},
                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                            {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                            {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'ReferencedConsignmentID', MIN: 0, MAX: 99999},
                                {ID: cbc+'TransportEquipmentTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'ProviderTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'OwnerTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'SizeTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'DispositionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigerationOnIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                {ID: cbc+'ReturnabilityIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'LegalStatusIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'AirFlowPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'HumidityPercent', MIN: 0, MAX: 1},
                                {ID: cbc+'AnimalFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'HumanFoodApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'DangerousGoodsApprovedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'RefrigeratedIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'Characteristics', MIN: 0, MAX: 1},
                                {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cbc+'SpecialTransportRequirements', MIN: 0, MAX: 99999},
                                {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TareWeightMeasure', MIN: 0, MAX: 1},
                                {ID: cbc+'TrackingDeviceCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PowerIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 0, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                    {ID: cbc+'MinimumMeasure', MIN: 0, MAX: 1},
                                    {ID: cbc+'MaximumMeasure', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'TransportEquipmentSeal', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                    {ID: cbc+'SealIssuerTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'Condition', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealStatusCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'SealingPartyType', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                    {ID: cbc+'Measure', MIN: 1, MAX: 1},
                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                {ID: cac+'ProviderParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingProofParty', MIN: 0, MAX: 1},
                                {ID: cac+'SupplierParty', MIN: 0, MAX: 1},
                                {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                {ID: cac+'OperatingParty', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingLocation', MIN: 0, MAX: 1},
                                {ID: cac+'StorageLocation', MIN: 0, MAX: 1},
                                {ID: cac+'PositioningTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'QuarantineTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PickupTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'HandlingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ApplicableTransportMeans', MIN: 0, MAX: 1, LEVEL: [
                                    {ID: cbc+'JourneyID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationalityID', MIN: 0, MAX: 1},
                                    {ID: cbc+'RegistrationNationality', MIN: 0, MAX: 99999},
                                    {ID: cbc+'DirectionCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TradeServiceCode', MIN: 0, MAX: 1},
                                    {ID: cac+'Stowage', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LocationID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Location', MIN: 0, MAX: 99999},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    ]},
                                    {ID: cac+'AirTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'AircraftID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RoadTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'LicensePlateID', MIN: 1, MAX: 1},
                                    ]},
                                    {ID: cac+'RailTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'TrainID', MIN: 1, MAX: 1},
                                        {ID: cbc+'RailCarID', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'MaritimeTransport', MIN: 0, MAX: 1, LEVEL: [
                                        {ID: cbc+'VesselID', MIN: 0, MAX: 1},
                                        {ID: cbc+'VesselName', MIN: 0, MAX: 1},
                                        {ID: cbc+'RadioCallSignID', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShipsRequirements', MIN: 0, MAX: 99999},
                                        {ID: cbc+'GrossTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cbc+'NetTonnageMeasure', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryCertificateDocumentReference', MIN: 0, MAX: 1},
                                        {ID: cac+'RegistryPortLocation', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'OwnerParty', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                ]},
                                {ID: cac+'HaulageTradingTerms', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'Information', MIN: 0, MAX: 99999},
                                    {ID: cbc+'Reference', MIN: 0, MAX: 1},
                                    {ID: cac+'ApplicableAddress', MIN: 0, MAX: 1},
                                ]},
                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'TransportEmergencyCardCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'PackingCriteriaCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HazardousRegulationCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'InhalationToxicityZoneCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportAuthorizationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                ]},
                                {ID: cac+'PackagedTransportHandlingUnit', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                    {ID: cbc+'TransportHandlingUnitTypeCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'HandlingInstructions', MIN: 0, MAX: 99999},
                                    {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalGoodsItemQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'TotalPackageQuantity', MIN: 0, MAX: 1},
                                    {ID: cbc+'DamageRemarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'ShippingMarks', MIN: 0, MAX: 99999},
                                    {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                    {ID: cac+'HandlingUnitDespatchLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'DeliveredQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'BackorderReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OutstandingQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'OutstandingReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 1, MAX: 99999, LEVEL: [
                                            {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                            {ID: cbc+'SalesOrderLineID', MIN: 0, MAX: 1},
                                            {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                            {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                            {ID: cac+'OrderReference', MIN: 0, MAX: 1},
                                        ]},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                                            {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'Name', MIN: 0, MAX: 1},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                            {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                                            {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                                            {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                                            {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                                                {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                                                {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                                            {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                                            {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                                                {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                                                {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                                                {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                                                {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                                                {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                                                    {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                                {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                                {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            ]},
                                            {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                                            {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Name', MIN: 1, MAX: 1},
                                                {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                                                {ID: cbc+'Value', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                                                {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                                                {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                                {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                                                    {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                                                ]},
                                            ]},
                                            {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                                            {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                                            {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                                                {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                                                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                                                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                                                {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                                                {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                                                {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'Signature', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        ]},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'ActualPackage', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReturnableMaterialIndicator', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackageLevelCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackagingTypeCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'PackingMaterial', MIN: 0, MAX: 99999},
                                        {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                        {ID: cac+'ContainedPackage', MIN: 0, MAX: 99999},
                                        {ID: cac+'ContainingTransportEquipment', MIN: 0, MAX: 1},
                                        {ID: cac+'GoodsItem', MIN: 0, MAX: 99999, LEVEL: [
                                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                                            {ID: cbc+'SequenceNumberID', MIN: 0, MAX: 1},
                                            {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                            {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredCustomsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredForCarriageValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'DeclaredStatisticsValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'FreeOnBoardValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'InsuranceValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'ValueAmount', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetNetWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableWeightMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'GrossVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                                            {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'PreferenceCriterionCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'RequiredCustomsID', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsStatusCode', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsTariffQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'CustomsImportClassifiedIndicator', MIN: 0, MAX: 1},
                                            {ID: cbc+'ChargeableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'ReturnableQuantity', MIN: 0, MAX: 1},
                                            {ID: cbc+'TraceID', MIN: 0, MAX: 1},
                                            {ID: cac+'Item', MIN: 0, MAX: 99999},
                                            {ID: cac+'GoodsItemContainer', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                                                {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                            ]},
                                            {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                            {ID: cac+'InvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                                                {ID: cbc+'ID', MIN: 1, MAX: 1},
                                                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                                                {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                                                {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                                                {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                                                {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                                                {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                                                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                                                    {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                                                ]},
                                                {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                                {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                                                        {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                                                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                                                        {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                                                        {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                                                            {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                                                            {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                                                            {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                                                            {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                                                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                            {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                                                {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                                                            ]},
                                                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                            {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                                                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                                                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                                                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                                                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                                                {ID: cbc+'Date', MIN: 0, MAX: 1},
                                                                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                                                            ]},
                                                        ]},
                                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                                        {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                                                        {ID: cac+'Package', MIN: 0, MAX: 1},
                                                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                        {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                                                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                                                            {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                                                            {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                                                        ]},
                                                    ]},
                                                    {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                                                ]},
                                                {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                                                {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                                                {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                                                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                                                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                                                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                                                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                                                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                                                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                                ]},
                                                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                                                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                                                {ID: cac+'Item', MIN: 1, MAX: 1},
                                                {ID: cac+'Price', MIN: 0, MAX: 1},
                                                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                                                {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                                                {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                                                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                                                    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                                                ]},
                                            ]},
                                            {ID: cac+'Temperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'ContainedGoodsItem', MIN: 0, MAX: 99999},
                                            {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                                            {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                            {ID: cac+'Pickup', MIN: 0, MAX: 1, LEVEL: [
                                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'ActualPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'EarliestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupDate', MIN: 0, MAX: 1},
                                                {ID: cbc+'LatestPickupTime', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupLocation', MIN: 0, MAX: 1},
                                                {ID: cac+'PickupParty', MIN: 0, MAX: 1},
                                            ]},
                                            {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                            {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                            {ID: cac+'ContainingPackage', MIN: 0, MAX: 99999},
                                            {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 1},
                                            {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                            {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                        ]},
                                        {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                                        {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                        {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                        {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                    ]},
                                    {ID: cac+'ReceivedHandlingUnitReceiptLine', MIN: 0, MAX: 99999, LEVEL: [
                                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                                        {ID: cbc+'UUID', MIN: 0, MAX: 1},
                                        {ID: cbc+'Note', MIN: 0, MAX: 99999},
                                        {ID: cbc+'ReceivedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ShortageActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectedQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReasonCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'RejectReason', MIN: 0, MAX: 99999},
                                        {ID: cbc+'RejectActionCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'QuantityDiscrepancyCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'OversupplyQuantity', MIN: 0, MAX: 1},
                                        {ID: cbc+'ReceivedDate', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaintCode', MIN: 0, MAX: 1},
                                        {ID: cbc+'TimingComplaint', MIN: 0, MAX: 1},
                                        {ID: cac+'OrderLineReference', MIN: 0, MAX: 1},
                                        {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                                        {ID: cac+'Item', MIN: 0, MAX: 99999},
                                        {ID: cac+'Shipment', MIN: 0, MAX: 99999},
                                    ]},
                                    {ID: cac+'TransportEquipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'TransportMeans', MIN: 0, MAX: 99999},
                                    {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'MinimumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'MaximumTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                                    {ID: cac+'FloorSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'PalletSpaceMeasurementDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                                    {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                    {ID: cac+'Status', MIN: 0, MAX: 99999},
                                    {ID: cac+'CustomsDeclaration', MIN: 0, MAX: 99999},
                                    {ID: cac+'ReferencedShipment', MIN: 0, MAX: 99999},
                                    {ID: cac+'Package', MIN: 0, MAX: 99999},
                                ]},
                                {ID: cac+'ServiceAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'AttachedTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Delivery', MIN: 0, MAX: 1},
                                {ID: cac+'Pickup', MIN: 0, MAX: 1},
                                {ID: cac+'Despatch', MIN: 0, MAX: 1},
                                {ID: cac+'ShipmentDocumentReference', MIN: 0, MAX: 99999},
                                {ID: cac+'ContainedInTransportEquipment', MIN: 0, MAX: 99999},
                                {ID: cac+'Package', MIN: 0, MAX: 99999},
                                {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                            ]},
                            {ID: cac+'SupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'UnsupportedTransportEquipment', MIN: 0, MAX: 99999},
                            {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'SupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'UnsupportedCommodityClassification', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'CommodityClassificationType')},
                            {ID: cac+'TotalCapacityDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                            {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportModeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransportMeansTypeCode', MIN: 0, MAX: 1},
                                {ID: cbc+'TransitDirectionCode', MIN: 0, MAX: 1},
                                {ID: cbc+'PreCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'OnCarriageIndicator', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'EstimatedDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryDate', MIN: 0, MAX: 1},
                                {ID: cbc+'RequiredDeliveryTime', MIN: 0, MAX: 1},
                                {ID: cbc+'LoadingSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'SuccessiveSequenceID', MIN: 0, MAX: 1},
                                {ID: cbc+'Instructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'DemurrageInstructions', MIN: 0, MAX: 99999},
                                {ID: cbc+'CrewQuantity', MIN: 0, MAX: 1},
                                {ID: cbc+'PassengerQuantity', MIN: 0, MAX: 1},
                                {ID: cac+'TransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'CarrierParty', MIN: 0, MAX: 99999},
                                {ID: cac+'TransportMeans', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'UnloadingPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'TransshipPortLocation', MIN: 0, MAX: 1},
                                {ID: cac+'LoadingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExaminationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AvailabilityTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ExportationTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DischargeTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'WarehousingTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'OptionalTakeoverTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DropoffTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualPickupTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'DeliveryTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ReceiptTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'StorageTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'AcceptanceTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TerminalOperatorParty', MIN: 0, MAX: 1},
                                {ID: cac+'CustomsAgentParty', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedTransitPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                                {ID: cac+'FreightChargeLocation', MIN: 0, MAX: 1},
                                {ID: cac+'DetentionTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'RequestedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'RequestedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'PlannedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PlannedWaypointTransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'ActualDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualWaypointTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'ActualArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                                {ID: cac+'EstimatedDepartureTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'EstimatedArrivalTransportEvent', MIN: 0, MAX: 1},
                                {ID: cac+'PassengerPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'DriverPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'ReportingPerson', MIN: 0, MAX: 1},
                                {ID: cac+'CrewMemberPerson', MIN: 0, MAX: 99999},
                                {ID: cac+'SecurityOfficerPerson', MIN: 0, MAX: 1},
                                {ID: cac+'MasterPerson', MIN: 0, MAX: 1},
                                {ID: cac+'ShipsSurgeonPerson', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'TransportEvent', MIN: 0, MAX: 99999},
                            {ID: cac+'ResponsibleTransportServiceProviderParty', MIN: 0, MAX: 1},
                            {ID: cac+'EnvironmentalEmission', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'EnvironmentalEmissionTypeCode', MIN: 1, MAX: 1},
                                {ID: cbc+'ValueMeasure', MIN: 1, MAX: 1},
                                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                                {ID: cac+'EmissionCalculationMethod', MIN: 0, MAX: 99999, LEVEL: [
                                    {ID: cbc+'CalculationMethodCode', MIN: 0, MAX: 1},
                                    {ID: cbc+'FullnessIndicationCode', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementFromLocation', MIN: 0, MAX: 1},
                                    {ID: cac+'MeasurementToLocation', MIN: 0, MAX: 1},
                                ]},
                            ]},
                            {ID: cac+'EstimatedDurationPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'ScheduledServiceFrequency', MIN: 0, MAX: 99999, LEVEL: [
                                {ID: cbc+'WeekDayCode', MIN: 1, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'FinalDeliveryTransportationService', MIN: 0, MAX: 1},
                        {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'CollectPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'DisbursementPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'PrepaidPaymentTerms', MIN: 0, MAX: 1},
                        {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'ExtraAllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'MainCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'PreCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'OnCarriageShipmentStage', MIN: 0, MAX: 99999},
                        {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                        {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                        {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'GoodsItem', MIN: 0, MAX: 99999},
                    {ID: cac+'ShipmentStage', MIN: 0, MAX: 99999},
                    {ID: cac+'Delivery', MIN: 0, MAX: 1},
                    {ID: cac+'TransportHandlingUnit', MIN: 0, MAX: 99999},
                    {ID: cac+'ReturnAddress', MIN: 0, MAX: 1},
                    {ID: cac+'OriginAddress', MIN: 0, MAX: 1},
                    {ID: cac+'FirstArrivalPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'LastExitPortLocation', MIN: 0, MAX: 1},
                    {ID: cac+'ExportCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                    {ID: cac+'FreightAllowanceCharge', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                {ID: cbc+'Amount', MIN: 0, MAX: 1},
                {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
            ]},
            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'ChargeIndicator', MIN: 1, MAX: 1},
                {ID: cbc+'AllowanceChargeReasonCode', MIN: 0, MAX: 1},
                {ID: cbc+'AllowanceChargeReason', MIN: 0, MAX: 99999},
                {ID: cbc+'MultiplierFactorNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'PrepaidIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'SequenceNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                {ID: cbc+'BaseAmount', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                {ID: cac+'TaxCategory', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReasonCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxExemptionReason', MIN: 0, MAX: 99999},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxScheme', MIN: 1, MAX: 1},
                ]},
                {ID: cac+'TaxTotal', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                    {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                        {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'Percent', MIN: 0, MAX: 1},
                        {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                        {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                        {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                    ]},
                ]},
                {ID: cac+'PaymentMeans', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansCode', MIN: 1, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentChannelCode', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionID', MIN: 0, MAX: 1},
                    {ID: cbc+'InstructionNote', MIN: 0, MAX: 99999},
                    {ID: cbc+'PaymentID', MIN: 0, MAX: 99999},
                    {ID: cac+'CardAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'PrimaryAccountNumberID', MIN: 1, MAX: 1},
                        {ID: cbc+'NetworkID', MIN: 1, MAX: 1},
                        {ID: cbc+'CardTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ValidityStartDate', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cbc+'IssuerID', MIN: 0, MAX: 1},
                        {ID: cbc+'IssueNumberID', MIN: 0, MAX: 1},
                        {ID: cbc+'CV2ID', MIN: 0, MAX: 1},
                        {ID: cbc+'CardChipCode', MIN: 0, MAX: 1},
                        {ID: cbc+'ChipApplicationID', MIN: 0, MAX: 1},
                        {ID: cbc+'HolderName', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'PayeeFinancialAccount', MIN: 0, MAX: 1},
                    {ID: cac+'CreditAccount', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'AccountID', MIN: 1, MAX: 1},
                    ]},
                    {ID: cac+'PaymentMandate', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'MandateTypeCode', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaymentInstructionsNumeric', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumPaidAmount', MIN: 0, MAX: 1},
                        {ID: cbc+'SignatureID', MIN: 0, MAX: 1},
                        {ID: cac+'PayerParty', MIN: 0, MAX: 1},
                        {ID: cac+'PayerFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'PaymentReversalPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: [
                            {ID: cbc+'ID', MIN: 0, MAX: 1},
                            {ID: cbc+'Content', MIN: 0, MAX: 99999},
                        ]},
                    ]},
                    {ID: cac+'TradeFinancing', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'FinancingInstrumentCode', MIN: 0, MAX: 1},
                        {ID: cac+'ContractDocumentReference', MIN: 0, MAX: 1},
                        {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                        {ID: cac+'FinancingParty', MIN: 1, MAX: 1},
                        {ID: cac+'FinancingFinancialAccount', MIN: 0, MAX: 1},
                        {ID: cac+'Clause', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'ClauseType')},
                    ]},
                ]},
            ]},
            {ID: cac+'TaxTotal', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                ]},
            ]},
            {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                ]},
            ]},
            {ID: cac+'Item', MIN: 1, MAX: 1, LEVEL: [
                {ID: cbc+'Description', MIN: 0, MAX: 99999},
                {ID: cbc+'PackQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'PackSizeNumeric', MIN: 0, MAX: 1},
                {ID: cbc+'CatalogueIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'Name', MIN: 0, MAX: 1},
                {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                {ID: cbc+'Keyword', MIN: 0, MAX: 99999},
                {ID: cbc+'BrandName', MIN: 0, MAX: 99999},
                {ID: cbc+'ModelName', MIN: 0, MAX: 99999},
                {ID: cac+'BuyersItemIdentification', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'ExtendedID', MIN: 0, MAX: 1},
                    {ID: cbc+'BarcodeSymbologyID', MIN: 0, MAX: 1},
                    {ID: cac+'PhysicalAttribute', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'AttributeID', MIN: 1, MAX: 1},
                        {ID: cbc+'PositionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'DescriptionCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'MeasurementDimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
                    {ID: cac+'IssuerParty', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'SellersItemIdentification', MIN: 0, MAX: 1},
                {ID: cac+'ManufacturersItemIdentification', MIN: 0, MAX: 99999},
                {ID: cac+'StandardItemIdentification', MIN: 0, MAX: 1},
                {ID: cac+'CatalogueItemIdentification', MIN: 0, MAX: 1},
                {ID: cac+'AdditionalItemIdentification', MIN: 0, MAX: 99999},
                {ID: cac+'CatalogueDocumentReference', MIN: 0, MAX: 1},
                {ID: cac+'ItemSpecificationDocumentReference', MIN: 0, MAX: 99999},
                {ID: cac+'OriginCountry', MIN: 0, MAX: 1, LEVEL: sequence(cac+'CountryType')},
                {ID: cac+'CommodityClassification', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'NatureCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CargoTypeCode', MIN: 0, MAX: 1},
                    {ID: cbc+'CommodityCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ItemClassificationCode', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'TransactionConditions', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'ActionCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Description', MIN: 0, MAX: 99999},
                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'HazardousItem', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                    {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                    {ID: cbc+'AdditionalInformation', MIN: 0, MAX: 99999},
                    {ID: cbc+'UNDGCode', MIN: 0, MAX: 1},
                    {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                    {ID: cbc+'MedicalFirstAidGuideCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TechnicalName', MIN: 0, MAX: 1},
                    {ID: cbc+'CategoryName', MIN: 0, MAX: 1},
                    {ID: cbc+'HazardousCategoryCode', MIN: 0, MAX: 1},
                    {ID: cbc+'UpperOrangeHazardPlacardID', MIN: 0, MAX: 1},
                    {ID: cbc+'LowerOrangeHazardPlacardID', MIN: 0, MAX: 1},
                    {ID: cbc+'MarkingID', MIN: 0, MAX: 1},
                    {ID: cbc+'HazardClassID', MIN: 0, MAX: 1},
                    {ID: cbc+'NetWeightMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'NetVolumeMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'Quantity', MIN: 0, MAX: 1},
                    {ID: cac+'ContactParty', MIN: 0, MAX: 1},
                    {ID: cac+'SecondaryHazard', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 0, MAX: 1},
                        {ID: cbc+'PlacardNotation', MIN: 0, MAX: 1},
                        {ID: cbc+'PlacardEndorsement', MIN: 0, MAX: 1},
                        {ID: cbc+'EmergencyProceduresCode', MIN: 0, MAX: 1},
                        {ID: cbc+'Extension', MIN: 0, MAX: 99999},
                    ]},
                    {ID: cac+'HazardousGoodsTransit', MIN: 0, MAX: 99999},
                    {ID: cac+'EmergencyTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                    {ID: cac+'FlashpointTemperature', MIN: 0, MAX: 1, LEVEL: sequence(cac+'TemperatureType')},
                    {ID: cac+'AdditionalTemperature', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'TemperatureType')},
                ]},
                {ID: cac+'ClassifiedTaxCategory', MIN: 0, MAX: 99999},
                {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'Name', MIN: 1, MAX: 1},
                    {ID: cbc+'NameCode', MIN: 0, MAX: 1},
                    {ID: cbc+'TestMethod', MIN: 0, MAX: 1},
                    {ID: cbc+'Value', MIN: 0, MAX: 1},
                    {ID: cbc+'ValueQuantity', MIN: 0, MAX: 1},
                    {ID: cbc+'ValueQualifier', MIN: 0, MAX: 99999},
                    {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                    {ID: cbc+'ListValue', MIN: 0, MAX: 99999},
                    {ID: cac+'UsabilityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'ItemPropertyGroup', MIN: 0, MAX: 99999, LEVEL: [
                        {ID: cbc+'ID', MIN: 1, MAX: 1},
                        {ID: cbc+'Name', MIN: 0, MAX: 1},
                        {ID: cbc+'ImportanceCode', MIN: 0, MAX: 1},
                    ]},
                    {ID: cac+'RangeDimension', MIN: 0, MAX: 1, LEVEL: sequence(cac+'DimensionType')},
                    {ID: cac+'ItemPropertyRange', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'MinimumValue', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumValue', MIN: 0, MAX: 1},
                    ]},
                ]},
                {ID: cac+'ManufacturerParty', MIN: 0, MAX: 99999},
                {ID: cac+'InformationContentProviderParty', MIN: 0, MAX: 1},
                {ID: cac+'OriginAddress', MIN: 0, MAX: 99999},
                {ID: cac+'ItemInstance', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ProductTraceID', MIN: 0, MAX: 1},
                    {ID: cbc+'ManufactureDate', MIN: 0, MAX: 1},
                    {ID: cbc+'ManufactureTime', MIN: 0, MAX: 1},
                    {ID: cbc+'BestBeforeDate', MIN: 0, MAX: 1},
                    {ID: cbc+'RegistrationID', MIN: 0, MAX: 1},
                    {ID: cbc+'SerialID', MIN: 0, MAX: 1},
                    {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                    {ID: cac+'LotIdentification', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'LotNumberID', MIN: 0, MAX: 1},
                        {ID: cbc+'ExpiryDate', MIN: 0, MAX: 1},
                        {ID: cac+'AdditionalItemProperty', MIN: 0, MAX: 99999},
                    ]},
                ]},
                {ID: cac+'Certificate', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 1, MAX: 1},
                    {ID: cbc+'CertificateTypeCode', MIN: 1, MAX: 1},
                    {ID: cbc+'CertificateType', MIN: 1, MAX: 1},
                    {ID: cbc+'Remarks', MIN: 0, MAX: 99999},
                    {ID: cac+'IssuerParty', MIN: 1, MAX: 1},
                    {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                    {ID: cac+'Signature', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'Dimension', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DimensionType')},
            ]},
            {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                    {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                    {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                    {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                    {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                    {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                    {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                    {ID: cbc+'Date', MIN: 0, MAX: 1},
                    {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                ]},
            ]},
            {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'ID', MIN: 0, MAX: 1},
                {ID: cbc+'SpecialTerms', MIN: 0, MAX: 99999},
                {ID: cbc+'LossRiskResponsibilityCode', MIN: 0, MAX: 1},
                {ID: cbc+'LossRisk', MIN: 0, MAX: 99999},
                {ID: cbc+'Amount', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryLocation', MIN: 0, MAX: 1},
                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 1},
            ]},
            {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'ID', MIN: 1, MAX: 1},
                {ID: cbc+'UUID', MIN: 0, MAX: 1},
                {ID: cbc+'Note', MIN: 0, MAX: 99999},
                {ID: cbc+'InvoicedQuantity', MIN: 0, MAX: 1},
                {ID: cbc+'LineExtensionAmount', MIN: 1, MAX: 1},
                {ID: cbc+'TaxPointDate', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCostCode', MIN: 0, MAX: 1},
                {ID: cbc+'AccountingCost', MIN: 0, MAX: 1},
                {ID: cbc+'PaymentPurposeCode', MIN: 0, MAX: 1},
                {ID: cbc+'FreeOfChargeIndicator', MIN: 0, MAX: 1},
                {ID: cac+'InvoicePeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                {ID: cac+'OrderLineReference', MIN: 0, MAX: 99999},
                {ID: cac+'DespatchLineReference', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'LineID', MIN: 1, MAX: 1},
                    {ID: cbc+'UUID', MIN: 0, MAX: 1},
                    {ID: cbc+'LineStatusCode', MIN: 0, MAX: 1},
                    {ID: cac+'DocumentReference', MIN: 0, MAX: 1},
                ]},
                {ID: cac+'ReceiptLineReference', MIN: 0, MAX: 99999},
                {ID: cac+'BillingReference', MIN: 0, MAX: 99999},
                {ID: cac+'DocumentReference', MIN: 0, MAX: 99999},
                {ID: cac+'PricingReference', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cac+'OriginalItemLocationQuantity', MIN: 0, MAX: 1, LEVEL: [
                        {ID: cbc+'LeadTimeMeasure', MIN: 0, MAX: 1},
                        {ID: cbc+'MinimumQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'MaximumQuantity', MIN: 0, MAX: 1},
                        {ID: cbc+'HazardousRiskIndicator', MIN: 0, MAX: 1},
                        {ID: cbc+'TradingRestrictions', MIN: 0, MAX: 99999},
                        {ID: cac+'ApplicableTerritoryAddress', MIN: 0, MAX: 99999},
                        {ID: cac+'Price', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'PriceAmount', MIN: 1, MAX: 1},
                            {ID: cbc+'BaseQuantity', MIN: 0, MAX: 1},
                            {ID: cbc+'PriceChangeReason', MIN: 0, MAX: 99999},
                            {ID: cbc+'PriceTypeCode', MIN: 0, MAX: 1},
                            {ID: cbc+'PriceType', MIN: 0, MAX: 1},
                            {ID: cbc+'OrderableUnitFactorRate', MIN: 0, MAX: 1},
                            {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                            {ID: cac+'PriceList', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'ID', MIN: 0, MAX: 1},
                                {ID: cbc+'StatusCode', MIN: 0, MAX: 1},
                                {ID: cac+'ValidityPeriod', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'PeriodType')},
                                {ID: cac+'PreviousPriceList', MIN: 0, MAX: 1},
                            ]},
                            {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                            {ID: cac+'PricingExchangeRate', MIN: 0, MAX: 1, LEVEL: [
                                {ID: cbc+'SourceCurrencyCode', MIN: 1, MAX: 1},
                                {ID: cbc+'SourceCurrencyBaseRate', MIN: 0, MAX: 1},
                                {ID: cbc+'TargetCurrencyCode', MIN: 1, MAX: 1},
                                {ID: cbc+'TargetCurrencyBaseRate', MIN: 0, MAX: 1},
                                {ID: cbc+'ExchangeMarketID', MIN: 0, MAX: 1},
                                {ID: cbc+'CalculationRate', MIN: 0, MAX: 1},
                                {ID: cbc+'MathematicOperatorCode', MIN: 0, MAX: 1},
                                {ID: cbc+'Date', MIN: 0, MAX: 1},
                                {ID: cac+'ForeignExchangeContract', MIN: 0, MAX: 1},
                            ]},
                        ]},
                        {ID: cac+'DeliveryUnit', MIN: 0, MAX: 99999, LEVEL: sequence(cac+'DeliveryUnitType')},
                        {ID: cac+'ApplicableTaxCategory', MIN: 0, MAX: 99999},
                        {ID: cac+'Package', MIN: 0, MAX: 1},
                        {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                        {ID: cac+'DependentPriceReference', MIN: 0, MAX: 1, LEVEL: [
                            {ID: cbc+'Percent', MIN: 0, MAX: 1},
                            {ID: cac+'LocationAddress', MIN: 0, MAX: 1},
                            {ID: cac+'DependentLineReference', MIN: 0, MAX: 1},
                        ]},
                    ]},
                    {ID: cac+'AlternativeConditionPrice', MIN: 0, MAX: 99999},
                ]},
                {ID: cac+'OriginatorParty', MIN: 0, MAX: 1},
                {ID: cac+'Delivery', MIN: 0, MAX: 99999},
                {ID: cac+'PaymentTerms', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'ID', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentMeansID', MIN: 0, MAX: 99999},
                    {ID: cbc+'PrepaidPaymentReferenceID', MIN: 0, MAX: 1},
                    {ID: cbc+'Note', MIN: 0, MAX: 99999},
                    {ID: cbc+'ReferenceEventCode', MIN: 0, MAX: 1},
                    {ID: cbc+'SettlementDiscountPercent', MIN: 0, MAX: 1},
                    {ID: cbc+'PenaltySurchargePercent', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentPercent', MIN: 0, MAX: 1},
                    {ID: cbc+'Amount', MIN: 0, MAX: 1},
                    {ID: cbc+'SettlementDiscountAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'PenaltyAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentTermsDetailsURI', MIN: 0, MAX: 1},
                    {ID: cbc+'PaymentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'InstallmentDueDate', MIN: 0, MAX: 1},
                    {ID: cbc+'InvoicingPartyReference', MIN: 0, MAX: 1},
                    {ID: cac+'SettlementPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'PenaltyPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                    {ID: cac+'ExchangeRate', MIN: 0, MAX: 1},
                    {ID: cac+'ValidityPeriod', MIN: 0, MAX: 1, LEVEL: sequence(cac+'PeriodType')},
                ]},
                {ID: cac+'AllowanceCharge', MIN: 0, MAX: 99999},
                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                {ID: cac+'WithholdingTaxTotal', MIN: 0, MAX: 99999},
                {ID: cac+'Item', MIN: 1, MAX: 1},
                {ID: cac+'Price', MIN: 0, MAX: 1},
                {ID: cac+'DeliveryTerms', MIN: 0, MAX: 1},
                {ID: cac+'SubInvoiceLine', MIN: 0, MAX: 99999},
                {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                    {ID: cbc+'Amount', MIN: 1, MAX: 1},
                    {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
                ]},
            ]},
            {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
                {ID: cbc+'Amount', MIN: 1, MAX: 1},
                {ID: cac+'TaxTotal', MIN: 0, MAX: 99999},
            ]},
        ]},
        {ID: cac+'ItemPriceExtension', MIN: 0, MAX: 1, LEVEL: [
            {ID: cbc+'Amount', MIN: 1, MAX: 1},
            {ID: cac+'TaxTotal', MIN: 0, MAX: 99999, LEVEL: [
                {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                {ID: cbc+'RoundingAmount', MIN: 0, MAX: 1},
                {ID: cbc+'TaxEvidenceIndicator', MIN: 0, MAX: 1},
                {ID: cbc+'TaxIncludedIndicator', MIN: 0, MAX: 1},
                {ID: cac+'TaxSubtotal', MIN: 0, MAX: 99999, LEVEL: [
                    {ID: cbc+'TaxableAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TaxAmount', MIN: 1, MAX: 1},
                    {ID: cbc+'CalculationSequenceNumeric', MIN: 0, MAX: 1},
                    {ID: cbc+'TransactionCurrencyTaxAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'Percent', MIN: 0, MAX: 1},
                    {ID: cbc+'BaseUnitMeasure', MIN: 0, MAX: 1},
                    {ID: cbc+'PerUnitAmount', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRange', MIN: 0, MAX: 1},
                    {ID: cbc+'TierRatePercent', MIN: 0, MAX: 1},
                    {ID: cac+'TaxCategory', MIN: 1, MAX: 1},
                ]},
            ]},
        ]},
    ]},
]},
]
